/**
 * This script creates the 'grammartest' TinyMCE plugin with the related 'selectgrammartest' TinyMCE button, more information
 * are available in the TinyMCE documentation:
 *
 * - https://www.tinymce.com/docs/
 *
 * The modal window is handled with the windowManager API:
 *
 * - https://www.tinymce.com/docs/api/tinymce/tinymce.windowmanager/
 *
 * To have a general idea on how this script works check out the following resources:
 *
 * - https://www.tinymce.com/docs/advanced/creating-a-plugin/
 * - http://jamesdigioia.com/add-button-pop-wordpresss-tinymce-editor/
 */
(function($) {

  //Creates the 'grammartest' plugin
  tinymce.PluginManager.add('grammartest', function(editor, url) {

    //Creates the 'selectgrammartest' TinyMCE button and associate the 'opengrammartest' command with its click
    editor.addButton('selectgrammartest',
        {title: 'Grammar Test', cmd: 'opengrammartest', icon: 'icon dashicons-welcome-learn-more',});

    //Create the 'opengrammartest' command
    editor.addCommand('opengrammartest', function() {

      //Open the modal window
      editor.windowManager.open({

        //Set the various properties of the modal window
        title: 'Grammar Test',
        width: 300,
        height: 72,
        id: 'dagt-grammartest',

        /**
         * Add buttons in the bottom of the modal window. (eg. Add Table, Add Shortcode, Insert, Cancel, etc.)
         *
         * The buttons should be provided as objects of this array, in this case there is only a single object/button.
         */
        buttons: [
          {
            text: 'Add Test',
            id: 'dagt-button-add-test',
            class: 'dalt-button'
            ,
            onclick: function(e) {
              var testId = $('#test-id').val();
              if (testId === null) {
                testId = '';
              }

              //Add the shortcode in the post
              editor.execCommand('mceInsertContent', false, '[grammar-test id="' + testId + '"]');

              //Close the modal window
              editor.windowManager.close();
            },
          }],

      });

      //Generates the content of modal window
      appendContent();

    });

  });

  /**
   * Get the modal window content with an AJAX request and append it in the modal window.
   */
  function appendContent() {

    //Get a reference to the DOM element
    var contentBody = $('#dagt-grammartest-body');

    //Prepare the parameters for the AJAX request
    var data = {
      action: 'dagt_get_modal_window_content',
      security: dagt_nonce,
    };

    //Send the AJAX request
    $.post(ajaxurl, data, function(responseHtml) {

      //Append the content in the modal window
      $(contentBody).append(responseHtml);

      //Initialize Chosen
      $('#test-id').chosen();

    });

  }

})(jQuery);