<?php

/*
 * this class should be used to work with the administrative side of wordpress
 */
class Dagt_Admin{

    protected static $instance = null;
    private $shared = null;

    private $screen_id_tests = null;
    private $screen_id_sections = null;
    private $screen_id_students = null;
	private $screen_id_help = null;
    private $screen_id_options = null;

    private function __construct() {

        //assign an instance of the plugin info
        $this->shared = Dagt_Shared::get_instance();

        //Load admin stylesheets and JavaScript
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_styles' ) );
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );

        //Write in back end head
        add_action('admin_head', array( $this, 'wr_admin_head' ));

        //Add the admin menu
        add_action( 'admin_menu', array( $this, 'me_add_admin_menu' ) );

        //Load the options API registrations and callbacks
        add_action('admin_init', array( $this, 'op_register_options' ) );

        //Export CSV controller
        add_action('init', array($this, 'export_csv_controller'));

        //this hook is triggered during the creation of a new blog
        add_action('wpmu_new_blog', array($this, 'new_blog_create_options_and_tables'), 10, 6);

        //this hook is triggered during the deletion of a blog
        add_action( 'delete_blog', array($this, 'delete_blog_delete_options_and_tables'), 10, 1 );

        //Create tinymce plugin
        add_action('init', array( $this, 'create_tinymce_plugin') );
        
        //After an user is deleted delete the user results
        add_action( 'deleted_user', array( $this, 'delete_user_results') );

    }

    /*
     * return an istance of this class
     */
    public static function get_instance() {

        if ( null == self::$instance ) {
            self::$instance = new self;
        }

        return self::$instance;

    }

    /*
     * write in the admin head
     */
    public function wr_admin_head(){

        echo '<script type="text/javascript">';
            echo 'var dagt_ajax_url = "' . admin_url('admin-ajax.php') . '";';
            echo 'var dagt_nonce = "' . wp_create_nonce( "dagt" ) . '";';
            echo 'var dagt_admin_url ="' . get_admin_url() . '";';
        echo '</script>';

    }

    public function enqueue_admin_styles() {

        $screen = get_current_screen();

        //menu tests
        if ( $screen->id == $this->screen_id_tests ) {
            wp_enqueue_style( $this->shared->get('slug') .'-framework-menu', $this->shared->get('url') . 'admin/assets/css/framework/menu.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-menu-test', $this->shared->get('url') . 'admin/assets/css/menu-test.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-chosen-custom', $this->shared->get('url') . 'admin/assets/css/chosen-custom.css', array(), $this->shared->get('ver') );
	        wp_enqueue_style( $this->shared->get('slug') .'-jquery-ui-tooltip', $this->shared->get('url') . 'admin/assets/css/jquery-ui-tooltip.css', array(), $this->shared->get('ver') );
        }

        //menu sections
        if ( $screen->id == $this->screen_id_sections ) {
            wp_enqueue_style( $this->shared->get('slug') .'-framework-menu', $this->shared->get('url') . 'admin/assets/css/framework/menu.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-menu-section', $this->shared->get('url') . 'admin/assets/css/menu-section.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-chosen-custom', $this->shared->get('url') . 'admin/assets/css/chosen-custom.css', array(), $this->shared->get('ver') );
	        wp_enqueue_style( $this->shared->get('slug') .'-jquery-ui-tooltip', $this->shared->get('url') . 'admin/assets/css/jquery-ui-tooltip.css', array(), $this->shared->get('ver') );
        }

        //menu students
        if ( $screen->id == $this->screen_id_students ) {
            wp_enqueue_style( $this->shared->get('slug') .'-framework-menu', $this->shared->get('url') . 'admin/assets/css/framework/menu.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-menu-students', $this->shared->get('url') . 'admin/assets/css/menu-students.css', array(), $this->shared->get('ver') );
	        wp_enqueue_style( $this->shared->get('slug') .'-jquery-ui-tooltip', $this->shared->get('url') . 'admin/assets/css/jquery-ui-tooltip.css', array(), $this->shared->get('ver') );
        }

	    //Menu Help
	    if ( $screen->id == $this->screen_id_help ) {
		    wp_enqueue_style( $this->shared->get( 'slug' ) . '-menu-help',
			    $this->shared->get( 'url' ) . 'admin/assets/css/menu-help.css', array(), $this->shared->get( 'ver' ) );
	    }

        //menu options
        if ( $screen->id == $this->screen_id_options ) {
            wp_enqueue_style( 'wp-color-picker' );
            wp_enqueue_style( $this->shared->get('slug') .'-framework-options', $this->shared->get('url') . 'admin/assets/css/framework/options.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-chosen-custom', $this->shared->get('url') . 'admin/assets/css/chosen-custom.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-jquery-ui-tooltip', $this->shared->get('url') . 'admin/assets/css/jquery-ui-tooltip.css', array(), $this->shared->get('ver') );
            wp_enqueue_style( $this->shared->get('slug') .'-menu-options', $this->shared->get('url') . 'admin/assets/css/menu-options.css', array(), $this->shared->get('ver') );
        }

        //post editor
        $args = array(
            'show_ui' => true
        );
        $post_types_with_ui = get_post_types($args);
        unset($post_types_with_ui['attachment']);

        if (in_array($screen->id, $post_types_with_ui) and current_user_can(get_option( $this->shared->get('slug') . "_tests_menu_capability"))) {

            /*
             * This enables the use of custom icons from Dashicons without using a .png image.
             *
             * For details: https://www.gavick.com/blog/wordpress-tinymce-custom-buttons#tc-section-4
             */
            wp_enqueue_style($this->shared->get('slug') . '-enable-dashicons-in-tinymce', $this->shared->get('url') . 'admin/assets/css/enable-dashicons-in-tinymce.css', array(), $this->shared->get('ver'));

            //Load Chosen CSS
            wp_enqueue_style( $this->shared->get( 'slug' ) . '-chosen', $this->shared->get( 'url' ) . 'shared/assets/inc/chosen/chosen.css', array(), $this->shared->get( 'ver' ) );

            //Load the CSS used in the modal window opened when the TinyMCE Grammar Test button is clicked
            wp_enqueue_style( $this->shared->get( 'slug' ) . '-tinymce-modal-window', $this->shared->get( 'url' ) . 'admin/assets/css/tinymce-grammartest.css', array(), $this->shared->get( 'ver' ) );

        }

    }

    /*
     * enqueue admin-specific javascript
     */
    public function enqueue_admin_scripts() {

        $screen = get_current_screen();

        //menu tests
        if ( $screen->id == $this->screen_id_tests ) {
            wp_enqueue_script( $this->shared->get('slug') . '-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.js', 'jquery', $this->shared->get('ver') );
            wp_enqueue_script( $this->shared->get('slug') . '-menu-test', $this->shared->get('url') . 'admin/assets/js/menu-test.js', 'jquery', $this->shared->get('ver') );
	        wp_enqueue_script('jquery-ui-tooltip');
	        wp_enqueue_script( $this->shared->get('slug') . '-jquery-ui-tooltip-init', $this->shared->get('url') . 'admin/assets/js/jquery-ui-tooltip-init.js', 'jquery', $this->shared->get('ver') );
        }

        //menu sections
        if ( $screen->id == $this->screen_id_sections ) {
            wp_enqueue_script( $this->shared->get('slug') . '-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.js', 'jquery', $this->shared->get('ver') );
            wp_enqueue_script( $this->shared->get('slug') . '-menu-section', $this->shared->get('url') . 'admin/assets/js/menu-section.js', 'jquery', $this->shared->get('ver') );
	        wp_enqueue_script('jquery-ui-tooltip');
	        wp_enqueue_script( $this->shared->get('slug') . '-jquery-ui-tooltip-init', $this->shared->get('url') . 'admin/assets/js/jquery-ui-tooltip-init.js', 'jquery', $this->shared->get('ver') );
        }

        //menu students
        if ( $screen->id == $this->screen_id_students ) {
            wp_enqueue_script( $this->shared->get('slug') . '-menu-students', $this->shared->get('url') . 'admin/assets/js/menu-students.js', 'jquery', $this->shared->get('ver') );
	        wp_enqueue_script('jquery-ui-tooltip');
	        wp_enqueue_script( $this->shared->get('slug') . '-jquery-ui-tooltip-init', $this->shared->get('url') . 'admin/assets/js/jquery-ui-tooltip-init.js', 'jquery', $this->shared->get('ver') );
        }

        //menu options
        if( $screen->id == $this->screen_id_options ){

            wp_enqueue_script( $this->shared->get('slug') . '-menu-options', $this->shared->get('url') . 'admin/assets/js/menu-options.js', array( 'wp-color-picker' ), false, true );
            wp_enqueue_script( $this->shared->get('slug') . '-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.js', 'jquery', $this->shared->get('ver') );
            wp_enqueue_script('jquery-ui-tooltip');
            wp_enqueue_script( $this->shared->get('slug') . '-jquery-ui-tooltip-init', $this->shared->get('url') . 'admin/assets/js/jquery-ui-tooltip-init.js', 'jquery', $this->shared->get('ver') );
            
        }

        //Enqueue scripts in all the post types of the post editor
        $args = array(
            'show_ui' => true
        );
        $post_types_with_ui = get_post_types($args);
        unset($post_types_with_ui['attachment']);

        if ( in_array( $screen->id, $post_types_with_ui ) and current_user_can(get_option( $this->shared->get('slug') . "_tests_menu_capability")) ) {

            //Load the Chosen JS
            wp_enqueue_script( $this->shared->get('slug') . '-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.js', 'jquery', $this->shared->get('ver') );

        }

    }

    /*
     * plugin activation
     */
    public function ac_activate($networkwide){
        
        /*
         * Verify the multisite environment
         */
        if(function_exists('is_multisite') and is_multisite()) {

            /*
             * if this is a "Network Activation" create the options and tables
             * for each blog
             */
            if ($networkwide) {

                //get the current blog id
                global $wpdb;
                $current_blog = $wpdb->blogid;

                //create an array with all the blog ids
                $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");

                //iterate through all the blogs
                foreach ($blogids as $blog_id){

                    //swith to the iterated blog
                    switch_to_blog($blog_id);

                    //create options and tables for the iterated blog
                    $this->ac_initialize_options();
                    $this->ac_create_database_tables();
                    $this->ac_create_student_role();
                    $this->ac_initialize_custom_css();

                }

                //switch to the current blog
                switch_to_blog($current_blog);

            }else{

                /*
                 * if this is not a "Network Activation" create options and
                 * tables only for the current blog
                 */
                $this->ac_initialize_options();
                $this->ac_create_database_tables();
                $this->ac_create_student_role();
                $this->ac_initialize_custom_css();

            }

        }else{

            /*
             * if this is not a multisite installation create options and
             * tables only for the current blog
             */
            $this->ac_initialize_options();
            $this->ac_create_database_tables();
            $this->ac_create_student_role();
            $this->ac_initialize_custom_css();

        }
    }

    //create the options and tables for the newly created blog
    public function new_blog_create_options_and_tables($blog_id, $user_id, $domain, $path, $site_id, $meta ) {

        global $wpdb;

        /*
         * if the plugin is "Network Active" create the options and tables for
         * this new blog
         */
        if(is_plugin_active_for_network('grammar-test/init.php')){

            //get the id of the current blog
            $current_blog = $wpdb->blogid;

            //switch to the blog that is being activated
            switch_to_blog($blog_id);

            //create options and database tables for the new blog
            $this->ac_initialize_options();
            $this->ac_create_database_tables();
            $this->ac_create_student_role();
            $this->ac_initialize_custom_css();

            //switch to the current blog
            switch_to_blog($current_blog);

        }

    }

    //delete options and tables for the deleted blog
    public function delete_blog_delete_options_and_tables($blog_id){

        global $wpdb;

        //get the id of the current blog
        $current_blog = $wpdb->blogid;

        //switch to the blog that is being activated
        switch_to_blog($blog_id);

        //create options and database tables for the new blog
        $this->un_delete_options();
        $this->un_delete_database_tables();

        //switch to the current blog
        switch_to_blog($current_blog);

    }

    /*
     * initialize plugin options
     */
    private function ac_initialize_options(){

        //database version -----------------------------------------------------
        add_option( $this->shared->get('slug') . "_database_version","0");

        //general options ------------------------------------------------------
        add_option( $this->shared->get('slug') . "_base_font_size","13");
        add_option( $this->shared->get('slug') . "_base_line_height","21");
        add_option( $this->shared->get('slug') . "_test_top_margin","21");
        add_option( $this->shared->get('slug') . "_test_bottom_margin","21");
        add_option( $this->shared->get('slug') . "_input_horizontal_margin","2");
        add_option( $this->shared->get('slug') . "_select_horizontal_margin","2");
        add_option( $this->shared->get('slug') . "_base_font_weight","400");
        add_option( $this->shared->get('slug') . "_headings_font_weight","600");
        add_option( $this->shared->get('slug') . "_section_description_font_weight","400");
        add_option( $this->shared->get('slug') . "_enumeration_font_weight","400");
        add_option( $this->shared->get('slug') . "_corrected_answers_font_weight","400");
        add_option( $this->shared->get('slug') . "_submit_button_font_weight","400");
        add_option( $this->shared->get('slug') . "_base_font_family","'Open Sans', Helvetica, Arial, sans-serif");
        add_option( $this->shared->get('slug') . "_headings_font_family","'Open Sans', Helvetica, Arial, sans-serif");
        add_option( $this->shared->get('slug') . "_correct_answers_decoration","2");
        add_option( $this->shared->get('slug') . "_wrong_answers_decoration","2");
        
        //color options --------------------------------------------------------
        add_option( $this->shared->get('slug') . "_base_font_color","#666666");
        add_option( $this->shared->get('slug') . "_test_title_font_color","#222222");
        add_option( $this->shared->get('slug') . "_section_title_font_color","#00794a");
        add_option( $this->shared->get('slug') . "_section_description_font_color","#666666");
        add_option( $this->shared->get('slug') . "_enumeration_font_color","#666666");
        add_option( $this->shared->get('slug') . "_input_font_color","#666666");
        add_option( $this->shared->get('slug') . "_input_border_color","#e1e1e1");
        add_option( $this->shared->get('slug') . "_select_font_color","#666666");
        add_option( $this->shared->get('slug') . "_select_background_color","#fff");
        add_option( $this->shared->get('slug') . "_select_border_color","#e1e1e1");
        add_option( $this->shared->get('slug') . "_select_arrow_color","#666666");
        add_option( $this->shared->get('slug') . "_select_highlight_background_color","#f8f8f8");
        add_option( $this->shared->get('slug') . "_select_highlight_font_color","#666666");
        add_option( $this->shared->get('slug') . "_correct_answers_color","#00794a");
        add_option( $this->shared->get('slug') . "_wrong_answers_color","#666666");
        add_option( $this->shared->get('slug') . "_tooltip_font_color","#666666");
        add_option( $this->shared->get('slug') . "_tooltip_background_color","#fff");
        add_option( $this->shared->get('slug') . "_tooltip_border_color","#e1e1e1");
        add_option( $this->shared->get('slug') . "_submit_button_font_color","#fff");
        add_option( $this->shared->get('slug') . "_submit_button_background_color","#00794a");
        add_option( $this->shared->get('slug') . "_submit_button_background_color_hover","#00794a");
        add_option( $this->shared->get('slug') . "_ajax_loader_color","#00794a");
        add_option( $this->shared->get('slug') . "_messages_font_color","#666666");
        add_option( $this->shared->get('slug') . "_messages_background_color","#fcfcfc");
        add_option( $this->shared->get('slug') . "_messages_border_color","#e1e1e1");
        
        //labels options
        add_option( $this->shared->get('slug') . "_submit_button_text", esc_attr__('Check Answers', 'dagt'));
        add_option( $this->shared->get('slug') . "_reset_button_text", esc_attr__('Repeat Test', 'dagt'));
        add_option( $this->shared->get('slug') . "_results_title_text", esc_attr__('Evaluation and Answers', 'dagt'));
        add_option( $this->shared->get('slug') . "_evaluation_text_part_1", esc_attr__('You answered', 'dagt'));
        add_option( $this->shared->get('slug') . "_evaluation_text_part_2", esc_attr__('out of', 'dagt'));
        add_option( $this->shared->get('slug') . "_evaluation_text_part_3", esc_attr__('question correctly. Hold the mouse pointer over the corrected answers in order to get an explanation.', 'dagt'));
        add_option( $this->shared->get('slug') . "_section_text_part_1", esc_attr__('In this section you answered', 'dagt'));
        add_option( $this->shared->get('slug') . "_section_text_part_2", esc_attr__('out of', 'dagt'));
        add_option( $this->shared->get('slug') . "_section_text_part_3", esc_attr__('questions correctly.', 'dagt'));
        add_option( $this->shared->get('slug') . "_non_public_test_text", esc_attr__('This test is available only to the registered students.', 'dagt'));
        add_option( $this->shared->get('slug') . "_submitted_examination_text", esc_attr__('Thank you for completing the examination.', 'dagt'));
        
        //misc options
        add_option( $this->shared->get('slug') . "_show_test_title","1");
	    add_option( $this->shared->get('slug') . "_mw_multiple_correct_answer","0");
	    add_option( $this->shared->get('slug') . "_mw_multiple_correct_answer_separator",",");
	    add_option( $this->shared->get('slug') . "_mw_multiple_correct_answer_displayed_separator","/");
        add_option( $this->shared->get('slug') . "_remove_parentheses","1");
        add_option( $this->shared->get('slug') . "_auto_scroll","1");
        add_option( $this->shared->get('slug') . "_scroll_easing","easeOutQuart");
        add_option( $this->shared->get('slug') . "_test_scroll_offset","0");
        add_option( $this->shared->get('slug') . "_test_scroll_time","2000");
        add_option( $this->shared->get('slug') . "_random_input_offset","0");
        add_option( $this->shared->get('slug') . "_dashicon_size","20");
        add_option( $this->shared->get('slug') . "_explanation_dashicon","dashicons-admin-comments");
        add_option( $this->shared->get('slug') . "_load_google_font_1","https://fonts.googleapis.com/css?family=Open+Sans:400,600&subset=latin,greek,greek-ext,vietnamese,cyrillic-ext,latin-ext,cyrillic");
        add_option( $this->shared->get('slug') . "_load_google_font_2","");
        add_option( $this->shared->get('slug') . "_tests_menu_capability", "manage_options");
        add_option( $this->shared->get('slug') . "_sections_menu_capability", "manage_options");
        add_option( $this->shared->get('slug') . "_students_menu_capability", "manage_options");


    }

    /*
     * create the plugin database tables
     */
    private function ac_create_database_tables(){

        //check database version and create the database
        if( intval(get_option( $this->shared->get('slug') . '_database_version'), 10) < 1 ){

            require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

            //create *prefix*_test
            global $wpdb;
            $table_name = $wpdb->prefix . $this->shared->get('slug') . "_test";
            $sql = "CREATE TABLE $table_name (
                id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                title VARCHAR(255) DEFAULT '' NOT NULL,
                description VARCHAR(1000) DEFAULT '' NOT NULL,
                section1_id BIGINT DEFAULT 0 NOT NULL,
                section2_id BIGINT DEFAULT 0 NOT NULL,
                section3_id BIGINT DEFAULT 0 NOT NULL,
                section4_id BIGINT DEFAULT 0 NOT NULL,
                section5_id BIGINT DEFAULT 0 NOT NULL,
                section6_id BIGINT DEFAULT 0 NOT NULL,
                section7_id BIGINT DEFAULT 0 NOT NULL,
                section8_id BIGINT DEFAULT 0 NOT NULL,
                section9_id BIGINT DEFAULT 0 NOT NULL,
                section10_id BIGINT DEFAULT 0 NOT NULL,
                additional_info VARCHAR(1000) DEFAULT '' NOT NULL,
                type INT DEFAULT 1 NOT NULL

            )
            COLLATE = utf8_general_ci
            ";

            dbDelta($sql);

            //create *prefix*_section
            global $wpdb;
            $table_name = $wpdb->prefix . $this->shared->get('slug') . "_section";
            $sql = "CREATE TABLE $table_name (
                id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                title VARCHAR(255) DEFAULT '' NOT NULL,
                description VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence1 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence2 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence3 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence4 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence5 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence6 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence7 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence8 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence9 VARCHAR(1000) DEFAULT '' NOT NULL,
                sentence10 VARCHAR(1000) DEFAULT '' NOT NULL,
                enumerate TINYINT(1) DEFAULT 0 NOT NULL
            )
            COLLATE = utf8_general_ci
            ";

            dbDelta($sql);

            //create *prefix*_result
            global $wpdb;
            $table_name = $wpdb->prefix . $this->shared->get('slug') . "_result";
            $sql = "CREATE TABLE $table_name (
                id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                user_id BIGINT DEFAULT 0 NOT NULL,
                test_id BIGINT DEFAULT 0 NOT NULL,
                score_s1 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s2 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s3 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s4 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s5 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s6 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s7 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s8 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s9 TINYINT(1) DEFAULT 0 NOT NULL,
                score_s10 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s1 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s2 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s3 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s4 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s5 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s6 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s7 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s8 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s9 TINYINT(1) DEFAULT 0 NOT NULL,
                elements_s10 TINYINT(1) DEFAULT 0 NOT NULL,
                examination TINYINT(1) DEFAULT 0 NOT NULL,
                date DATETIME,
                date_gmt DATETIME
            )
            COLLATE = utf8_general_ci
            ";

            dbDelta($sql);

            //create *prefix*_missing_word
            global $wpdb;
            $table_name = $wpdb->prefix . $this->shared->get('slug') . "_missing_word";
            $sql = "CREATE TABLE $table_name (
                id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                correct_answer VARCHAR(255) DEFAULT '' NOT NULL,
                explanation VARCHAR(255) DEFAULT '' NOT NULL,
                case_insensitive TINYINT(1) DEFAULT 0 NOT NULL
            )
            COLLATE = utf8_general_ci
            ";

            dbDelta($sql);

            //create *prefix*_multiple_selection
            global $wpdb;
            $table_name = $wpdb->prefix . $this->shared->get('slug') . "_multiple_selection";
            $sql = "CREATE TABLE $table_name (
                id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
                alternative_1 VARCHAR(255) DEFAULT '' NOT NULL,
                alternative_2 VARCHAR(255) DEFAULT '' NOT NULL,
                alternative_3 VARCHAR(255) DEFAULT '' NOT NULL,
                alternative_4 VARCHAR(255) DEFAULT '' NOT NULL,
                alternative_5 VARCHAR(255) DEFAULT '' NOT NULL,
                correct_answer TINYINT(1) DEFAULT 0 NOT NULL,
                explanation VARCHAR(255) DEFAULT '' NOT NULL
            )
            COLLATE = utf8_general_ci
            ";

            dbDelta($sql);

            //Update database version
            update_option( $this->shared->get('slug') . '_database_version',"1");

        }

    }

    /*
     * create the "Student" role and assing its capabilities
     */
    private function ac_create_student_role(){

        add_role( 'dagt_student', 'Student', array(
            'dagt_perform_practice_and_examination' => true,
            'read' => true
        ));

    }
    
    /*
     * initialize the custom-[blog_id].css file
     */
    public function ac_initialize_custom_css(){
        
        /*
         * Write the custom-[blog_id].css file or die if the file can't be
         * created or modified
         */
        if( $this->shared->write_custom_css() === false ){
            die( 'The ' . esc_attr($this->shared->get('dir')) . 'public/assets/css/custom-' . get_current_blog_id() . '.css file should be writable by the server. Please change its permissions and try again.');
        }
        
    }

    /*
     * plugin delete
     */
    static public function un_delete(){

        /*
         * delete options and tables for all the sites in the network
         */
        if(function_exists('is_multisite') and is_multisite()) {

            //get the current blog id
            global $wpdb;
            $current_blog = $wpdb->blogid;

            //create an array with all the blog ids
            $blogids = $wpdb->get_col("SELECT blog_id FROM $wpdb->blogs");

            //iterate through all the blogs
            foreach ($blogids as $blog_id){

                //swith to the iterated blog
                switch_to_blog($blog_id);

                //create options and tables for the iterated blog
                Dagt_Admin::un_delete_options();
                Dagt_Admin::un_delete_database_tables();

            }

            //switch to the current blog
            switch_to_blog($current_blog);

        }else{

            /*
             * if this is not a multisite installation delete options and
             * tables only for the current blog
             */
            Dagt_Admin::un_delete_options();
            Dagt_Admin::un_delete_database_tables();

        }

    }

    /*
     * delete plugin options
     */
    static public function un_delete_options(){

        //assign an instance of Dagt_Shared
        $shared = Dagt_Shared::get_instance();

        //database version -----------------------------------------------------
        delete_option( $shared->get('slug') . "_database_version");

        //general options ------------------------------------------------------
        delete_option( $shared->get('slug') . "_base_font_size");
        delete_option( $shared->get('slug') . "_base_line_height");
        delete_option( $shared->get('slug') . "_test_top_margin");
        delete_option( $shared->get('slug') . "_test_bottom_margin");
        delete_option( $shared->get('slug') . "_input_horizontal_margin");
        delete_option( $shared->get('slug') . "_select_horizontal_margin");
        delete_option( $shared->get('slug') . "_base_font_weight");
        delete_option( $shared->get('slug') . "_headings_font_weight");
        delete_option( $shared->get('slug') . "_section_description_font_weight");
        delete_option( $shared->get('slug') . "_enumeration_font_weight");
        delete_option( $shared->get('slug') . "_corrected_answers_font_weight");
        delete_option( $shared->get('slug') . "_submit_button_font_weight");
        delete_option( $shared->get('slug') . "_base_font_family","'Open Sans', Helvetica, Arial, sans-serif");
        delete_option( $shared->get('slug') . "_headings_font_family","'Open Sans', Helvetica, Arial, sans-serif");
        delete_option( $shared->get('slug') . "_correct_answers_decoration");
        delete_option( $shared->get('slug') . "_wrong_answers_decoration");
        
        //color options --------------------------------------------------------
        delete_option( $shared->get('slug') . "_base_font_color");
        delete_option( $shared->get('slug') . "_test_title_font_color");
        delete_option( $shared->get('slug') . "_section_title_font_color");
        delete_option( $shared->get('slug') . "_section_description_font_color");
        delete_option( $shared->get('slug') . "_enumeration_font_color");
        delete_option( $shared->get('slug') . "_input_font_color");
        delete_option( $shared->get('slug') . "_input_border_color");
        delete_option( $shared->get('slug') . "_select_font_color");
        delete_option( $shared->get('slug') . "_select_background_color");
        delete_option( $shared->get('slug') . "_select_border_color");
        delete_option( $shared->get('slug') . "_select_arrow_color");
        delete_option( $shared->get('slug') . "_select_highlight_background_color");
        delete_option( $shared->get('slug') . "_select_highlight_font_color");
        delete_option( $shared->get('slug') . "_correct_answers_color");
        delete_option( $shared->get('slug') . "_wrong_answers_color");
        delete_option( $shared->get('slug') . "_tooltip_font_color");
        delete_option( $shared->get('slug') . "_tooltip_background_color");
        delete_option( $shared->get('slug') . "_tooltip_border_color");
        delete_option( $shared->get('slug') . "_submit_button_font_color");
        delete_option( $shared->get('slug') . "_submit_button_background_color");
        delete_option( $shared->get('slug') . "_submit_button_background_color_hover");
        delete_option( $shared->get('slug') . "_ajax_loader_color");
        delete_option( $shared->get('slug') . "_messages_font_color");
        delete_option( $shared->get('slug') . "_messages_background_color");
        delete_option( $shared->get('slug') . "_messages_border_color");
        
        //labels options
        delete_option( $shared->get('slug') . "_submit_button_text");
        delete_option( $shared->get('slug') . "_reset_button_text");
        delete_option( $shared->get('slug') . "_results_title_text");
        delete_option( $shared->get('slug') . "_evaluation_text_part_1");
        delete_option( $shared->get('slug') . "_evaluation_text_part_2");
        delete_option( $shared->get('slug') . "_evaluation_text_part_3");
        delete_option( $shared->get('slug') . "_section_text_part_1");
        delete_option( $shared->get('slug') . "_section_text_part_2");
        delete_option( $shared->get('slug') . "_section_text_part_3");
        delete_option( $shared->get('slug') . "_non_public_test_text");
        delete_option( $shared->get('slug') . "_submitted_examination_text");
        
        //misc options
        delete_option( $shared->get('slug') . "_show_test_title");
	    delete_option( $shared->get('slug') . "_mw_multiple_correct_answer");
	    delete_option( $shared->get('slug') . "_mw_multiple_correct_answer_separator");
	    delete_option( $shared->get('slug') . "_mw_multiple_correct_answer_displayed_separator");
        delete_option( $shared->get('slug') . "_remove_parentheses");
        delete_option( $shared->get('slug') . "_auto_scroll");
        delete_option( $shared->get('slug') . "_scroll_easing");
        delete_option( $shared->get('slug') . "_test_scroll_offset");
        delete_option( $shared->get('slug') . "_test_scroll_time");
        delete_option( $shared->get('slug') . "_random_input_offset");
        delete_option( $shared->get('slug') . "_dashicon_size");
        delete_option( $shared->get('slug') . "_explanation_dashicon");
        delete_option( $shared->get('slug') . "_load_google_font_1");
        delete_option( $shared->get('slug') . "_load_google_font_2");
        delete_option( $shared->get('slug') . "_tests_menu_capability");
        delete_option( $shared->get('slug') . "_sections_menu_capability");
        delete_option( $shared->get('slug') . "_students_menu_capability");

    }

    /*
     * delete plugin database tables
     */
    static public function un_delete_database_tables(){

        //assign an instance of Dagt_Shared
        $shared = Dagt_Shared::get_instance();

        global $wpdb;

        $table_name = $wpdb->prefix . $shared->get('slug') . "_test";
        $sql = "DROP TABLE $table_name";
        $wpdb->query($sql);
        
        $table_name = $wpdb->prefix . $shared->get('slug') . "_section";
        $sql = "DROP TABLE $table_name";
        $wpdb->query($sql);
        
        $table_name = $wpdb->prefix . $shared->get('slug') . "_result";
        $sql = "DROP TABLE $table_name";
        $wpdb->query($sql);
        
        $table_name = $wpdb->prefix . $shared->get('slug') . "_missing_word";
        $sql = "DROP TABLE $table_name";
        $wpdb->query($sql);
        
        $table_name = $wpdb->prefix . $shared->get('slug') . "_multiple_selection";
        $sql = "DROP TABLE $table_name";
        $wpdb->query($sql);

    }

    /*
     * register the admin menu
     */
    public function me_add_admin_menu() {

        add_menu_page(
            'Gt',
            'Grammar Test',
            get_option( $this->shared->get('slug') . "_tests_menu_capability"),
            $this->shared->get('slug') . '-tests',
            array( $this, 'me_display_menu_tests'),
            'dashicons-welcome-learn-more'
        );

        $this->screen_id_tests = add_submenu_page(
            $this->shared->get('slug') . '-tests',
            esc_attr__('GT - Tests', 'dagt'),
            esc_attr__('Tests', 'dagt'),
            get_option( $this->shared->get('slug') . "_tests_menu_capability"),
            $this->shared->get('slug') . '-tests',
            array( $this, 'me_display_menu_tests')
        );

        $this->screen_id_sections = add_submenu_page(
            $this->shared->get('slug') . '-tests',
            esc_attr__('GT - Sections', 'dagt'),
            esc_attr__('Sections', 'dagt'),
            get_option( $this->shared->get('slug') . "_sections_menu_capability"),
            $this->shared->get('slug') . '-sections',
            array( $this, 'me_display_menu_sections')
        );

        $this->screen_id_students = add_submenu_page(
            $this->shared->get('slug') . '-tests',
            esc_attr__('GT - Students', 'dagt'),
            esc_attr__('Students', 'dagt'),
            get_option( $this->shared->get('slug') . "_students_menu_capability"),
            $this->shared->get('slug') . '-students',
            array( $this, 'me_display_menu_students')
        );
		
	    $this->screen_id_help = add_submenu_page(
		    $this->shared->get('slug') . '-tests',
		    esc_html__('GT - Help', 'dagt'),
		    esc_html__('Help', 'dagt'),
		    'manage_options',
		    $this->shared->get('slug') . '_help',
		    array( $this, 'me_display_menu_help')
	    );

        $this->screen_id_options = add_submenu_page(
            $this->shared->get('slug') . '-tests',
            esc_attr__('GT - Options', 'dagt'),
            esc_attr__('Options', 'dagt'),
            'manage_options',
            $this->shared->get('slug') . '-options',
            array( $this, 'me_display_menu_options')
        );

    }

    /*
     * includes the tests view
     */
    public function me_display_menu_tests() {
        include_once( 'view/tests.php' );
    }

    /*
     * includes the sections view
     */
    public function me_display_menu_sections() {
        include_once( 'view/sections.php' );
    }

    /*
     * includes the students view
     */
    public function me_display_menu_students() {
        include_once( 'view/students.php' );
    }

	/*
	 * includes the help view
	 */
	public function me_display_menu_help() {
		include_once( 'view/help.php' );
	}

    /*
     * includes the options view
     */
    public function me_display_menu_options() {
        include_once( 'view/options.php' );
    }

    /*
     * register options
     */
    public function op_register_options() {

        //section typography ---------------------------------------------------
        add_settings_section(
            'dagt_typography_settings_section',
            NULL,
            NULL,
            'dagt_typography_options'
        );
        
        add_settings_field(
            'base_font_size',
            esc_attr__('Base Font Size (px)', 'dagt'),
            array($this,'base_font_size_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_base_font_size',
            array($this,'base_font_size_validation')
        );
        
        add_settings_field(
            'base_line_height',
            esc_attr__('Base Line Height (px)', 'dagt'),
            array($this,'base_line_height_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_base_line_height',
            array($this,'base_line_height_validation')
        );
        
        add_settings_field(
            'test_top_margin',
            esc_attr__('Test Top Margin (px)', 'dagt'),
            array($this,'test_top_margin_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_test_top_margin',
            array($this,'test_top_margin_validation')
        );
        
        add_settings_field(
            'test_bottom_margin',
            esc_attr__('Test Bottom Margin (px)', 'dagt'),
            array($this,'test_bottom_margin_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_test_bottom_margin',
            array($this,'test_bottom_margin_validation')
        );
        
        add_settings_field(
            'input_horizontal_margin',
            esc_attr__('Input Horizontal Margin', 'dagt'),
            array($this,'input_horizontal_margin_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_input_horizontal_margin',
            array($this,'input_horizontal_margin_validation')
        );
        
        add_settings_field(
            'select_horizontal_margin',
            esc_attr__('Select Horizontal Margin', 'dagt'),
            array($this,'select_horizontal_margin_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_select_horizontal_margin',
            array($this,'select_horizontal_margin_validation')
        );
        
        add_settings_field(
            'base_font_weight',
            esc_attr__('Base Font Weight', 'dagt'),
            array($this,'base_font_weight_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_base_font_weight',
            array($this,'base_font_weight_validation')
        );
        
        add_settings_field(
            'headings_font_weight',
            esc_attr__('Headings Font Weight', 'dagt'),
            array($this,'headings_font_weight_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_headings_font_weight',
            array($this,'headings_font_weight_validation')
        );
        
        add_settings_field(
            'section_description_font_weight',
            esc_attr__('Section Description Font Weight', 'dagt'),
            array($this,'section_description_font_weight_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_section_description_font_weight',
            array($this,'section_description_font_weight_validation')
        );
        
        add_settings_field(
            'enumeration_font_weight',
            esc_attr__('Enumeration Font Weight', 'dagt'),
            array($this,'enumeration_font_weight_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_enumeration_font_weight',
            array($this,'enumeration_font_weight_validation')
        );
        
        add_settings_field(
            'corrected_answers_font_weight',
            esc_attr__('Corrected Answers Font Weight', 'dagt'),
            array($this,'corrected_answers_font_weight_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_corrected_answers_font_weight',
            array($this,'corrected_answers_font_weight_validation')
        );
        
        add_settings_field(
            'submit_button_font_weight',
            esc_attr__('Submit Button Font Weight', 'dagt'),
            array($this,'submit_button_font_weight_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_submit_button_font_weight',
            array($this,'submit_button_font_weight_validation')
        ); 
        
        add_settings_field(
            'base_font_family',
            esc_attr__('Base Font Family', 'dagt'),
            array($this,'base_font_family_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_base_font_family',
            array($this,'base_font_family_validation')
        );
        
        add_settings_field(
            'headings_font_family',
            esc_attr__('Headings Font Family', 'dagt'),
            array($this,'headings_font_family_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_headings_font_family',
            array($this,'headings_font_family_validation')
        );
        
        add_settings_field(
            'correct_answers_decoration',
            esc_attr__('Correct Answers Decoration', 'dagt'),
            array($this,'correct_answers_decoration_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_correct_answers_decoration',
            array($this,'correct_answers_decoration_validation')
        );
        
        add_settings_field(
            'wrong_answers_decoration',
            esc_attr__('Wrong Answers Decoration', 'dagt'),
            array($this,'wrong_answers_decoration_callback'),
            'dagt_typography_options',
            'dagt_typography_settings_section'
        );

        register_setting(
            'dagt_typography_options',
            'dagt_wrong_answers_decoration',
            array($this,'wrong_answers_decoration_validation')
        );
        
        //section colors -------------------------------------------------------
        add_settings_section(
            'dagt_colors_settings_section',
            NULL,
            NULL,
            'dagt_colors_options'
        );

        add_settings_field(
            'base_font_color',
            esc_attr__('Base Font', 'dagt'),
            array($this,'base_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_base_font_color',
            array($this,'base_font_color_validation')
        );

        add_settings_field(
            'test_title_font_color',
            esc_attr__('Test Title Font', 'dagt'),
            array($this,'test_title_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_test_title_font_color',
            array($this,'test_title_font_color_validation')
        );

        add_settings_field(
            'section_title_font_color',
            esc_attr__('Section Title Font', 'dagt'),
            array($this,'section_title_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_section_title_font_color',
            array($this,'section_title_font_color_validation')
        );

        add_settings_field(
            'section_description_font_color',
            esc_attr__('Section Description Font', 'dagt'),
            array($this,'section_description_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_section_description_font_color',
            array($this,'section_description_font_color_validation')
        );
        
        add_settings_field(
            'enumeration_font_color',
            esc_attr__('Enumeration Font', 'dagt'),
            array($this,'enumeration_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_enumeration_font_color',
            array($this,'enumeration_font_color_validation')
        );
        
        add_settings_field(
            'input_font_color',
            esc_attr__('Input Font', 'dagt'),
            array($this,'input_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_input_font_color',
            array($this,'input_font_color_validation')
        );
        
        add_settings_field(
            'input_border_color',
            esc_attr__('Input Border', 'dagt'),
            array($this,'input_border_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_input_border_color',
            array($this,'input_border_color_validation')
        );
        
        add_settings_field(
            'select_font_color',
            esc_attr__('Select Font', 'dagt'),
            array($this,'select_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_select_font_color',
            array($this,'select_font_color_validation')
        );
        
        add_settings_field(
            'select_background_color',
            esc_attr__('Select Background', 'dagt'),
            array($this,'select_background_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_select_background_color',
            array($this,'select_background_color_validation')
        );
        
        add_settings_field(
            'select_border_color',
            esc_attr__('Select Border', 'dagt'),
            array($this,'select_border_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_select_border_color',
            array($this,'select_border_color_validation')
        );
        
        add_settings_field(
            'select_arrow_color',
            esc_attr__('Select Arrow', 'dagt'),
            array($this,'select_arrow_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_select_arrow_color',
            array($this,'select_arrow_color_validation')
        );
        
        add_settings_field(
            'select_highlight_background_color',
            esc_attr__('Select Highlight Background', 'dagt'),
            array($this,'select_highlight_background_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_select_highlight_background_color',
            array($this,'select_highlight_background_color_validation')
        );
        
        add_settings_field(
            'select_highlight_font_color',
            esc_attr__('Select Highlight Font', 'dagt'),
            array($this,'select_highlight_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_select_highlight_font_color',
            array($this,'select_highlight_font_color_validation')
        );
        
        add_settings_field(
            'correct_answers_color',
            esc_attr__('Correct Answers', 'dagt'),
            array($this,'correct_answers_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_correct_answers_color',
            array($this,'correct_answers_color_validation')
        );

        add_settings_field(
            'wrong_answers_color',
            esc_attr__('Wrong Answers', 'dagt'),
            array($this,'wrong_answers_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_wrong_answers_color',
            array($this,'wrong_answers_color_validation')
        );
        
        add_settings_field(
            'tooltip_font_color',
            esc_attr__('Tooltip Font', 'dagt'),
            array($this,'tooltip_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_tooltip_font_color',
            array($this,'tooltip_font_color_validation')
        );
        
        add_settings_field(
            'tooltip_background_color',
            esc_attr__('Tooltip Background', 'dagt'),
            array($this,'tooltip_background_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_tooltip_background_color',
            array($this,'tooltip_background_color_validation')
        );
        
        add_settings_field(
            'tooltip_border_color',
            esc_attr__('Tooltip Border', 'dagt'),
            array($this,'tooltip_border_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_tooltip_border_color',
            array($this,'tooltip_border_color_validation')
        );
        
        add_settings_field(
            'submit_button_font_color',
            esc_attr__('Submit Button Font', 'dagt'),
            array($this,'submit_button_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_submit_button_font_color',
            array($this,'submit_button_font_color_validation')
        );
        
        add_settings_field(
            'submit_button_background_color',
            esc_attr__('Submit Button Background', 'dagt'),
            array($this,'submit_button_background_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_submit_button_background_color',
            array($this,'submit_button_background_color_validation')
        );

        add_settings_field(
            'submit_button_background_color_hover',
            esc_attr__('Submit Button Background Hover', 'dagt'),
            array($this,'submit_button_background_color_hover_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_submit_button_background_color_hover',
            array($this,'submit_button_background_color_hover_validation')
        );
        
        add_settings_field(
            'ajax_loader_color',
            esc_attr__('Ajax Loader', 'dagt'),
            array($this,'ajax_loader_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_ajax_loader_color',
            array($this,'ajax_loader_color_validation')
        );

        add_settings_field(
            'messages_font_color',
            esc_attr__('Message Font', 'dagt'),
            array($this,'messages_font_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_messages_font_color',
            array($this,'messages_font_color_validation')
        );

        add_settings_field(
            'messages_background_color',
            esc_attr__('Message Background', 'dagt'),
            array($this,'messages_background_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_messages_background_color',
            array($this,'messages_background_color_validation')
        );
        
        add_settings_field(
            'messages_border_color',
            esc_attr__('Message Border', 'dagt'),
            array($this,'messages_border_color_callback'),
            'dagt_colors_options',
            'dagt_colors_settings_section'
        );

        register_setting(
            'dagt_colors_options',
            'dagt_messages_border_color',
            array($this,'messages_border_color_validation')
        );
        
        //section labels -------------------------------------------------------
        add_settings_section(
            'dagt_labels_settings_section',
            NULL,
            NULL,
            'dagt_labels_options'
        );

        add_settings_field(
            'submit_button_text',
            esc_attr__('Submit Button', 'dagt'),
            array($this,'submit_button_text_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_submit_button_text',
            array($this,'submit_button_text_validation')
        );
        
        add_settings_field(
            'reset_button_text',
            esc_attr__('Reset Button', 'dagt'),
            array($this,'reset_button_text_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_reset_button_text',
            array($this,'reset_button_text_validation')
        );
        
        add_settings_field(
            'results_title_text',
            esc_attr__('Results Title', 'dagt'),
            array($this,'results_title_text_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_results_title_text',
            array($this,'results_title_text_validation')
        );
        
        add_settings_field(
            'evaluation_text_part_1',
            esc_attr__('Evaluation Part 1', 'dagt'),
            array($this,'evaluation_text_part_1_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_evaluation_text_part_1',
            array($this,'evaluation_text_part_1_validation')
        );
        
        add_settings_field(
            'evaluation_text_part_2',
            esc_attr__('Evaluation Part 2', 'dagt'),
            array($this,'evaluation_text_part_2_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_evaluation_text_part_2',
            array($this,'evaluation_text_part_2_validation')
        );
        
        add_settings_field(
            'evaluation_text_part_3',
            esc_attr__('Evaluation Part 3', 'dagt'),
            array($this,'evaluation_text_part_3_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_evaluation_text_part_3',
            array($this,'evaluation_text_part_3_validation')
        );
        
        add_settings_field(
            'section_text_part_1',
            esc_attr__('Section Part 1', 'dagt'),
            array($this,'section_text_part_1_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_section_text_part_1',
            array($this,'section_text_part_1_validation')
        );
        
        add_settings_field(
            'section_text_part_2',
            esc_attr__('Section Part 2', 'dagt'),
            array($this,'section_text_part_2_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_section_text_part_2',
            array($this,'section_text_part_2_validation')
        );
        
        add_settings_field(
            'section_text_part_3',
            esc_attr__('Section Part 3', 'dagt'),
            array($this,'section_text_part_3_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_section_text_part_3',
            array($this,'section_text_part_3_validation')
        );
        
        add_settings_field(
            'non_public_test_text',
            esc_attr__('Non-Public Test', 'dagt'),
            array($this,'non_public_test_text_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_non_public_test_text',
            array($this,'non_public_test_text_validation')
        );
        
        add_settings_field(
            'submitted_examination_text',
            esc_attr__('Submitted Examination', 'dagt'),
            array($this,'submitted_examination_text_callback'),
            'dagt_labels_options',
            'dagt_labels_settings_section'
        );

        register_setting(
            'dagt_labels_options',
            'dagt_submitted_examination_text',
            array($this,'submitted_examination_text_validation')
        );

        //section advanced ---------------------------------------------------------
        add_settings_section(
            'dagt_advanced_settings_section',
            NULL,
            NULL,
            'dagt_advanced_options'
        );

        add_settings_field(
            'show_test_title',
            esc_attr__('Show Title', 'dagt'),
            array($this,'show_test_title_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_show_test_title',
            array($this,'show_test_title_validation')
        );

	    add_settings_field(
		    'mw_multiple_correct_answer',
		    esc_attr__('Multiple Answers', 'dagt'),
		    array($this,'mw_multiple_correct_answer_callback'),
		    'dagt_advanced_options',
		    'dagt_advanced_settings_section'
	    );

	    register_setting(
		    'dagt_advanced_options',
		    'dagt_mw_multiple_correct_answer',
		    array($this,'mw_multiple_correct_answer_validation')
	    );

	    add_settings_field(
		    'mw_multiple_correct_answer_separator',
		    esc_attr__('Multiple Answers Separator', 'dagt'),
		    array($this,'mw_multiple_correct_answer_separator_callback'),
		    'dagt_advanced_options',
		    'dagt_advanced_settings_section'
	    );

	    register_setting(
		    'dagt_advanced_options',
		    'dagt_mw_multiple_correct_answer_separator',
		    array($this,'mw_multiple_correct_answer_separator_validation')
	    );

	    add_settings_field(
		    'mw_multiple_correct_answer_displayed_separator',
		    esc_attr__('Multiple Answers Displayed Separator', 'dagt'),
		    array($this,'mw_multiple_correct_answer_displayed_separator_callback'),
		    'dagt_advanced_options',
		    'dagt_advanced_settings_section'
	    );

	    register_setting(
		    'dagt_advanced_options',
		    'dagt_mw_multiple_correct_answer_displayed_separator',
		    array($this,'mw_multiple_correct_answer_displayed_separator_validation')
	    );

        add_settings_field(
            'remove_parentheses',
            esc_attr__('Remove Parentheses', 'dagt'),
            array($this,'remove_parentheses_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_remove_parentheses',
            array($this,'remove_parentheses_validation')
        );
        
        add_settings_field(
            'auto_scroll',
            esc_attr__('Auto Scroll', 'dagt'),
            array($this,'auto_scroll_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_auto_scroll',
            array($this,'auto_scroll_validation')
        );
        
        add_settings_field(
            'scroll_easing',
            esc_attr__('Scroll Easing', 'dagt'),
            array($this,'scroll_easing_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_scroll_easing',
            array($this,'scroll_easing_validation')
        );
        
        add_settings_field(
            'test_scroll_offset',
            esc_attr__('Vertical Scroll Offset (px)', 'dagt'),
            array($this,'test_scroll_offset_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_test_scroll_offset',
            array($this,'test_scroll_offset_validation')
        );
        
        add_settings_field(
            'test_scroll_time',
            esc_attr__('Vertical Scroll Time (ms)', 'dagt'),
            array($this,'test_scroll_time_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_test_scroll_time',
            array($this,'test_scroll_time_validation')
        );
        
        add_settings_field(
            'random_input_offset',
            esc_attr__('Random Input Offset (px)', 'dagt'),
            array($this,'random_input_offset_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_random_input_offset',
            array($this,'random_input_offset_validation')
        );
        
        add_settings_field(
            'dashicon_size',
            esc_attr__('Dashicon Size', 'dagt'),
            array($this,'dashicon_size_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_dashicon_size',
            array($this,'dashicon_size_validation')
        );
        
        add_settings_field(
            'explanation_dashicon',
            esc_attr__('Explanation Dashicon', 'dagt'),
            array($this,'explanation_dashicon_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_explanation_dashicon',
            array($this,'explanation_dashicon_validation')
        );

        add_settings_field(
            'load_google_font_1',
            esc_attr__('Load Google Font 1', 'dagt'),
            array($this,'load_google_font_1_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_load_google_font_1',
            array($this,'load_google_font_1_validation')
        );
        
        add_settings_field(
            'load_google_font_2',
            esc_attr__('Load Google Font 2', 'dagt'),
            array($this,'load_google_font_2_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_load_google_font_2',
            array($this,'load_google_font_2_validation')
        );

        add_settings_field(
            'tests_menu_capability',
            esc_attr__('Tests Menu Capability', 'dagt'),
            array($this,'tests_menu_capability_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_tests_menu_capability',
            array($this,'tests_menu_capability_validation')
        );

        add_settings_field(
            'sections_menu_capability',
            esc_attr__('Sections Menu Capability', 'dagt'),
            array($this,'sections_menu_capability_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_sections_menu_capability',
            array($this,'sections_menu_capability_validation')
        );

        add_settings_field(
            'students_menu_capability',
            esc_attr__('Students Menu Capability', 'dagt'),
            array($this,'students_menu_capability_callback'),
            'dagt_advanced_options',
            'dagt_advanced_settings_section'
        );

        register_setting(
            'dagt_advanced_options',
            'dagt_students_menu_capability',
            array($this,'students_menu_capability_validation')
        );
        
    }

    //style options callbacks and validations ----------------------------------
    public function base_font_size_callback($args){

        $html = '<input type="text" id="dagt_base_font_size" name="dagt_base_font_size" class="regular-text" value="' . esc_attr(get_option("dagt_base_font_size")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__('This option is used to calculate the font size of the textual elements.', 'dagt')) . '"></div>';
        
        echo $html;

    }

    public function base_font_size_validation($input){

        if( intval($input, 10) < 1 or intval($input, 10) > 100 ){
            add_settings_error( 'dagt_base_font_size', 'dagt_base_font_size', esc_attr__('Please enter a number from 1 to 100 in the "Base Font Size" option.', 'dagt') );
            $output = get_option('dagt_base_font_size');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }

    public function base_line_height_callback($args){

        $html = '<input type="text" id="dagt_base_line_height" name="dagt_base_line_height" class="regular-text" value="' . esc_attr(get_option("dagt_base_line_height")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__('This option is used to calculate the line height of the textual elements and the height of input, select and button elements.', 'dagt')) . '"></div>';
        echo $html;

    }

    public function base_line_height_validation($input){

        if( intval($input, 10) < 1 or intval($input, 10) > 100 ){
            add_settings_error( 'dagt_base_line_height', 'dagt_base_line_height', esc_attr__('Please enter a number from 1 to 100 in the "Base Line Height" option.', 'dagt') );
            $output = get_option('dagt_base_line_height');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }

    public function base_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_base_font_color" name="' . $this->shared->get('slug') . '_base_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_base_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the textual elements that don't have a specific font color option.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function base_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'base_font_color', 'base_font_color', esc_attr__('Please enter a valid RGB color for the "Base Font Color" option.', 'dagt') );
            $output = get_option('dagt_base_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function base_font_weight_callback($args){

        $html = '<input type="text" id="dagt_base_font_weight" name="dagt_base_font_weight" class="regular-text" value="' . esc_attr(get_option("dagt_base_font_weight")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font weight of the textual elements that don't have a specific font weight option.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function base_font_weight_validation($input){

        if( intval($input, 10) < 100 or intval($input, 10) > 900 ){
            add_settings_error( 'dagt_base_font_weight', 'dagt_base_font_weight', esc_attr__('Please enter a number from 100 to 900 in the "Base Font Weight" option.', 'dagt') );
            $output = get_option('dagt_base_font_weight');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }

    public function base_font_family_callback($args) {

        $html = '<input maxlength="1000" type="text" id="' . $this->shared->get('slug') . '_base_font_family" name="' . $this->shared->get('slug') . '_base_font_family" class="regular-text" value="' . esc_attr(get_option("dagt_base_font_family")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font family of the textual elements that don't have a specific font family option.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function base_font_family_validation($input){

        if(!preg_match($this->shared->font_family_regex, $input)){
            add_settings_error( 'base_font_family', 'base_font_family', 'Please enter a valid font family in the "Base Font Family" option.', 'dagt' );
            $output = get_option('dagt_base_font_family');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function test_title_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_test_title_font_color" name="' . $this->shared->get('slug') . '_test_title_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_test_title_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the color of the test title.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function test_title_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'test_title_font_color', 'test_title_font_color', esc_attr__('Please enter a valid RGB color for the "Test Title Font Color" option.', 'dagt') );
            $output = get_option('dagt_test_title_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function headings_font_weight_callback($args){

        $html = '<input type="text" id="dagt_headings_font_weight" name="dagt_headings_font_weight" class="regular-text" value="' . esc_attr(get_option("dagt_headings_font_weight")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font weight of the headings.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function headings_font_weight_validation($input){

        if( intval($input, 10) < 100 or intval($input, 10) > 900 ){
            add_settings_error( 'dagt_headings_font_weight', 'dagt_headings_font_weight', esc_attr__('Please enter a number from 100 to 900 in the "Headings Font Weight" option.', 'dagt') );
            $output = get_option('dagt_headings_font_weight');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }

    public function headings_font_family_callback($args) {

        $html = '<input maxlength="1000" type="text" id="' . $this->shared->get('slug') . '_headings_font_family" name="' . $this->shared->get('slug') . '_headings_font_family" class="regular-text" value="' . esc_attr(get_option("dagt_headings_font_family")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font family of the headings.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function headings_font_family_validation($input){

        if(!preg_match($this->shared->font_family_regex, $input)){
            add_settings_error( 'headings_font_family', 'headings_font_family', 'Please enter a valid font family in the "Headings Font Family" option.', 'dagt' );
            $output = get_option('dagt_headings_font_family');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function submit_button_background_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_submit_button_background_color" name="' . $this->shared->get('slug') . '_submit_button_background_color" class="regular-text" value="' . esc_attr(get_option("dagt_submit_button_background_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the background color of the submit button.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function submit_button_background_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'submit_button_background_color', 'submit_button_background_color', esc_attr__('Please enter a valid RGB color for the "Submit Button Background Color" option.', 'dagt') );
            $output = get_option('dagt_submit_button_background_color');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function submit_button_background_color_hover_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_submit_button_background_color_hover" name="' . $this->shared->get('slug') . '_submit_button_background_color_hover" class="regular-text" value="' . esc_attr(get_option("dagt_submit_button_background_color_hover")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the background color of the submit button with the mouse over event.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function submit_button_background_color_hover_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'submit_button_background_color_hover', 'submit_button_background_color_hover', esc_attr__('Please enter a valid RGB color for the "Submit Button Background Color Hover" option.', 'dagt') );
            $output = get_option('dagt_submit_button_background_color_hover');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function ajax_loader_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_ajax_loader_color" name="' . $this->shared->get('slug') . '_ajax_loader_color" class="regular-text" value="' . esc_attr(get_option("dagt_ajax_loader_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the color of the ajax loader.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function ajax_loader_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'ajax_loader_color', 'ajax_loader_color', esc_attr__('Please enter a valid RGB color for the "Ajax Loader" option.', 'dagt') );
            $output = get_option('dagt_ajax_loader_color');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function submit_button_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_submit_button_font_color" name="' . $this->shared->get('slug') . '_submit_button_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_submit_button_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the submit button.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function submit_button_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'submit_button_font_color', 'submit_button_font_color', esc_attr__('Please enter a valid RGB color for the "Submit Button Font Color" option.', 'dagt') );
            $output = get_option('dagt_submit_button_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function correct_answers_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_correct_answers_color" name="' . $this->shared->get('slug') . '_correct_answers_color" class="regular-text" value="' . esc_attr(get_option("dagt_correct_answers_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the color of the correct answers.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function correct_answers_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'correct_answers_color', 'correct_answers_color', esc_attr__('Please enter a valid RGB color for the "Correct Answers Color" option.', 'dagt') );
            $output = get_option('dagt_correct_answers_color');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function wrong_answers_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_wrong_answers_color" name="' . $this->shared->get('slug') . '_wrong_answers_color" class="regular-text" value="' . esc_attr(get_option("dagt_wrong_answers_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the color of the wrong answers.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function wrong_answers_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'wrong_answers_color', 'wrong_answers_color', esc_attr__('Please enter a valid RGB color for the "Wrong Answers Color" option.', 'dagt') );
            $output = get_option('dagt_wrong_answers_color');
        }else{
            $output = $input;
        }

        return $output;

    }

    public function corrected_answers_font_weight_callback($args){

        $html = '<input type="text" id="dagt_corrected_answers_font_weight" name="dagt_corrected_answers_font_weight" class="regular-text" value="' . esc_attr(get_option("dagt_corrected_answers_font_weight")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font weight of the corrected answers.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function corrected_answers_font_weight_validation($input){

        if( intval($input, 10) < 100 or intval($input, 10) > 900 ){
            add_settings_error( 'dagt_corrected_answers_font_weight', 'dagt_corrected_answers_font_weight', esc_attr__('Please enter a number from 100 to 900 in the "Corrected Answers Font Weight" option.', 'dagt') );
            $output = get_option('dagt_corrected_answers_font_weight');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }

    public function messages_background_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_messages_background_color" name="' . $this->shared->get('slug') . '_messages_background_color" class="regular-text" value="' . esc_attr(get_option("dagt_messages_background_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the background color of the messages.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function messages_background_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'messages_background_color', 'messages_background_color', esc_attr__('Please enter a valid RGB color for the "Messages Background Color" option.', 'dagt') );
            $output = get_option('dagt_messages_background_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function messages_border_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_messages_border_color" name="' . $this->shared->get('slug') . '_messages_border_color" class="regular-text" value="' . esc_attr(get_option("dagt_messages_border_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the border color of the messages.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function messages_border_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'messages_border_color', 'messages_border_color', esc_attr__('Please enter a valid RGB color for the "Messages Border Color" option.', 'dagt') );
            $output = get_option('dagt_messages_border_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function messages_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_messages_font_color" name="' . $this->shared->get('slug') . '_messages_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_messages_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the messages.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function messages_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'messages_font_color', 'messages_font_color', esc_attr__('Please enter a valid RGB color for the "Messages Font Color" option.', 'dagt') );
            $output = get_option('dagt_messages_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function test_top_margin_callback($args){

        $html = '<input type="text" id="dagt_test_top_margin" name="dagt_test_top_margin" class="regular-text" value="' . esc_attr(get_option("dagt_test_top_margin")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the top margin of the test.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function test_top_margin_validation($input){

        if( intval($input, 10) > 1000 ){
            add_settings_error( 'dagt_test_top_margin', 'dagt_test_top_margin', esc_attr__('Please enter a number from 0 to 1000 in the "Test Top Margin" option.', 'dagt') );
            $output = get_option('dagt_test_top_margin');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function test_bottom_margin_callback($args){

        $html = '<input type="text" id="dagt_test_bottom_margin" name="dagt_test_bottom_margin" class="regular-text" value="' . esc_attr(get_option("dagt_test_bottom_margin")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the bottom margin of the test.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function test_bottom_margin_validation($input){

        if( intval($input, 10) > 1000 ){
            add_settings_error( 'dagt_test_bottom_margin', 'dagt_test_bottom_margin', esc_attr__('Please enter a number from 0 to 1000 in the "Test Bottom Margin" option.', 'dagt') );
            $output = get_option('dagt_test_bottom_margin');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function input_horizontal_margin_callback($args){

        $html = '<input type="text" id="dagt_input_horizontal_margin" name="dagt_input_horizontal_margin" class="regular-text" value="' . esc_attr(get_option("dagt_input_horizontal_margin")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the horizontal margin of the input element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function input_horizontal_margin_validation($input){

        if( intval($input, 10) > 1000 ){
            add_settings_error( 'dagt_input_horizontal_margin', 'dagt_input_horizontal_margin', esc_attr__('Please enter a number from 0 to 1000 in the "Input Horizontal Margin" option.', 'dagt') );
            $output = get_option('dagt_input_horizontal_margin');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function select_horizontal_margin_callback($args){

        $html = '<input type="text" id="dagt_select_horizontal_margin" name="dagt_select_horizontal_margin" class="regular-text" value="' . esc_attr(get_option("dagt_select_horizontal_margin")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the horizontal margin of the select element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function select_horizontal_margin_validation($input){

        if( intval($input, 10) > 1000 ){
            add_settings_error( 'dagt_select_horizontal_margin', 'dagt_select_horizontal_margin', esc_attr__('Please enter a number from 0 to 1000 in the "Select Horizontal Margin" option.', 'dagt') );
            $output = get_option('dagt_select_horizontal_margin');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function section_title_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_section_title_font_color" name="' . $this->shared->get('slug') . '_section_title_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_section_title_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the section title.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function section_title_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'section_title_font_color', 'section_title_font_color', esc_attr__('Please enter a valid RGB color for the "Section Title Font Color" option.', 'dagt') );
            $output = get_option('dagt_section_title_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function section_description_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_section_description_font_color" name="' . $this->shared->get('slug') . '_section_description_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_section_description_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the section description.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function section_description_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'section_description_font_color', 'section_description_font_color', esc_attr__('Please enter a valid RGB color for the "Section Description Font Color" option.', 'dagt') );
            $output = get_option('dagt_section_description_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function enumeration_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_enumeration_font_color" name="' . $this->shared->get('slug') . '_enumeration_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_enumeration_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the enumeration.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function enumeration_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'enumeration_font_color', 'enumeration_font_color', esc_attr__('Please enter a valid RGB color for the "Enumeration Font Color" option.', 'dagt') );
            $output = get_option('dagt_enumeration_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function section_description_font_weight_callback($args){

        $html = '<input type="text" id="dagt_section_description_font_weight" name="dagt_section_description_font_weight" class="regular-text" value="' . esc_attr(get_option("dagt_section_description_font_weight")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font weight of the section description.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function section_description_font_weight_validation($input){

        if( intval($input, 10) < 100 or intval($input, 10) > 900 ){
            add_settings_error( 'dagt_section_description_font_weight', 'dagt_section_description_font_weight', esc_attr__('Please enter a number from 100 to 900 in the "Section Description Font Weight" option.', 'dagt') );
            $output = get_option('dagt_section_description_font_weight');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function enumeration_font_weight_callback($args){

        $html = '<input type="text" id="dagt_enumeration_font_weight" name="dagt_enumeration_font_weight" class="regular-text" value="' . esc_attr(get_option("dagt_enumeration_font_weight")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font weight of the enumeration.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function enumeration_font_weight_validation($input){

        if( intval($input, 10) < 100 or intval($input, 10) > 900 ){
            add_settings_error( 'dagt_enumeration_font_weight', 'dagt_enumeration_font_weight', esc_attr__('Please enter a number from 100 to 900 in the "Enumeration Font Weight" option.', 'dagt') );
            $output = get_option('dagt_enumeration_font_weight');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function show_test_title_callback($args){

        $html = '<select class="daext-display-none" id="dagt_show_test_title" name="dagt_show_test_title">';
            $html .= '<option ' . selected(intval(get_option("dagt_show_test_title"), 10), 1, false) . ' value="1">' . esc_attr__('Yes', 'dagt') . '</option>';
            $html .= '<option ' . selected(intval(get_option("dagt_show_test_title"), 10), 0, false) . ' value="0">' . esc_attr__('No', 'dagt') . '</option>';
        $html .= '</select>';
        $html .= '<div class="help-icon" title="' . esc_attr(__('Select "Yes" to display the test title or "No" to hide the test title.', 'dagt')) . '"></div>';
        
        echo $html;

    }

    public function show_test_title_validation($input){

        return $input;

    }

	public function mw_multiple_correct_answer_callback($args){

		$html = '<select class="daext-display-none" id="dagt_mw_multiple_correct_answer" name="dagt_mw_multiple_correct_answer">';
		$html .= '<option ' . selected(intval(get_option("dagt_mw_multiple_correct_answer"), 10), 1, false) . ' value="1">' . esc_attr__('Yes', 'dagt') . '</option>';
		$html .= '<option ' . selected(intval(get_option("dagt_mw_multiple_correct_answer"), 10), 0, false) . ' value="0">' . esc_attr__('No', 'dagt') . '</option>';
		$html .= '</select>';
		$html .= '<div class="help-icon" title="' . esc_attr(__('Select "Yes" to enable the use of multiple correct answers with the missing words.', 'dagt')) . '"></div>';

		echo $html;

	}

	public function mw_multiple_correct_answer_validation($input){

		return $input;

	}

	public function mw_multiple_correct_answer_separator_callback($args){

		$html = '<input type="text" id="dagt_mw_multiple_correct_answer_separator" name="dagt_mw_multiple_correct_answer_separator" class="regular-text" value="' . esc_attr(get_option("dagt_mw_multiple_correct_answer_separator")) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr(__('Enter the separator that should be used in the "Correct Answer" field of the modal window used to add and edit the missing words.', 'dagt')) . '"></div>';
		echo $html;

	}

	public function mw_multiple_correct_answer_separator_validation($input){

		if( strlen(trim($input)) === 0 or strlen(trim($input)) > 255 ){
			add_settings_error( 'dagt_mw_multiple_correct_answer_separator', 'dagt_mw_multiple_correct_answer_separator', esc_attr__('Please enter a valid string in the "Multiple Answers Separator" option.', 'dagt') );
			$output = get_option('dagt_mw_multiple_correct_answer_separator');
		}else{
			$output = $input;
		}

		return $output;

	}

	public function mw_multiple_correct_answer_displayed_separator_callback($args){

		$html = '<input type="text" id="dagt_mw_multiple_correct_answer_displayed_separator" name="dagt_mw_multiple_correct_answer_displayed_separator" class="regular-text" value="' . esc_attr(get_option("dagt_mw_multiple_correct_answer_displayed_separator")) . '" />';
		$html .= '<div class="help-icon" title="' . esc_attr(__("Enter the string used to separate the missing words correct answers in the front-end.", 'dagt')) . '"></div>';
		echo $html;

	}

	public function mw_multiple_correct_answer_displayed_separator_validation($input){

		if( strlen(trim($input)) === 0 or strlen(trim($input)) > 255 ){
			add_settings_error( 'dagt_mw_multiple_correct_answer_displayed_separator', 'dagt_mw_multiple_correct_answer_displayed_separator', esc_attr__('Please enter a valid string in the "Multiple Answers Displayed Separator" option.', 'dagt') );
			$output = get_option('dagt_mw_multiple_correct_answer_displayed_separator');
		}else{
			$output = $input;
		}

		return $output;

	}

    public function remove_parentheses_callback($args){

        $html = '<select class="daext-display-none" id="dagt_remove_parentheses" name="dagt_remove_parentheses">';
            $html .= '<option ' . selected(intval(get_option("dagt_remove_parentheses"), 10), 1, false) . ' value="1">' . esc_attr__('Yes', 'dagt') . '</option>';
            $html .= '<option ' . selected(intval(get_option("dagt_remove_parentheses"), 10), 0, false) . ' value="0">' . esc_attr__('No', 'dagt') . '</option>';
        $html .= '</select>';
        $html .= '<div class="help-icon" title="' . esc_attr(__('Select "Yes" to remove the content included between round brackets from the corrected sentences.', 'dagt')) . '"></div>';
        
        echo $html;

    }

    public function remove_parentheses_validation($input){

        return $input;

    }
    
    public function auto_scroll_callback($args){

        $html = '<select class="daext-display-none" id="dagt_auto_scroll" name="dagt_auto_scroll">';
            $html .= '<option ' . selected(intval(get_option("dagt_auto_scroll"), 10), 1, false) . ' value="1">' . esc_attr__('Yes', 'dagt') . '</option>';
            $html .= '<option ' . selected(intval(get_option("dagt_auto_scroll"), 10), 0, false) . ' value="0">' . esc_attr__('No', 'dagt') . '</option>';
        $html .= '</select>';
        $html .= '<div class="help-icon" title="' . esc_attr(__('Select "Yes" to enable the auto scroll or "No" to disable it.', 'dagt')) . '"></div>';
        
        echo $html;

    }

    public function auto_scroll_validation($input){

        return $input;

    }
    
    public function scroll_easing_callback($args){

        $html = '<select class="daext-display-none" id="dagt_scroll_easing" name="dagt_scroll_easing">';
            
            $easing_list = 'swing,easeInQuad,easeOutQuad,easeInOutQuad,easeInCubic,easeOutCubic,easeInOutCubic,easeInQuart,easeOutQuart,easeInOutQuart,easeInQuint,easeOutQuint,easeInOutQuint,easeInSine,easeOutSine,easeInOutSine,easeInExpo,easeOutExpo,easeInOutExpo,easeInCirc,easeOutCirc,easeInOutCirc,easeInElastic,easeOutElastic,easeInOutElastic,easeInBack,easeOutBack,easeInOutBack,easeInBounce,easeOutBounce,easeInOutBounce';
            $easing_a = explode(',', $easing_list);
            
            foreach($easing_a as $key => $single_easing){
                
                $html .= '<option ' . selected(get_option("dagt_scroll_easing"), $single_easing, false) . ' value="' . $single_easing . '">' . $single_easing . '</option>';
                
            }

        $html .= '</select>';
        $html .= '<div class="help-icon" title="' . esc_attr(__('The scroll easing determines how the scroll speed is distributed over time.', 'dagt')) . '"></div>';
        
        echo $html;

    }

    public function scroll_easing_validation($input){

        return $input;

    }
    
    public function submit_button_font_weight_callback($args){

        $html = '<input type="text" id="dagt_submit_button_font_weight" name="dagt_submit_button_font_weight" class="regular-text" value="' . esc_attr(get_option("dagt_submit_button_font_weight")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font weight of the text included in the submit button.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function submit_button_font_weight_validation($input){

        if( intval($input, 10) < 100 or intval($input, 10) > 900 ){
            add_settings_error( 'dagt_submit_button_font_weight', 'dagt_submit_button_font_weight', esc_attr__('Please enter a number from 100 to 900 in the "Submit Button Font Weight" option.', 'dagt') );
            $output = get_option('dagt_submit_button_font_weight');
        }else{
            $output = $input;
        }
        
        return intval($output,  10);

    }
    
    public function input_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_input_font_color" name="' . $this->shared->get('slug') . '_input_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_input_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the input element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function input_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'input_font_color', 'input_font_color', esc_attr__('Please enter a valid RGB color for the "Input Font Color" option.', 'dagt') );
            $output = get_option('dagt_input_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function input_border_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_input_border_color" name="' . $this->shared->get('slug') . '_input_border_color" class="regular-text" value="' . esc_attr(get_option("dagt_input_border_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the border color of the input element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function input_border_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'input_border_color', 'input_border_color', esc_attr__('Please enter a valid RGB color for the "Input Border Color" option.', 'dagt') );
            $output = get_option('dagt_input_border_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function select_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_select_font_color" name="' . $this->shared->get('slug') . '_select_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_select_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the select element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function select_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'select_font_color', 'select_font_color', esc_attr__('Please enter a valid RGB color for the "Select Font Color" option.', 'dagt') );
            $output = get_option('dagt_select_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function select_background_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_select_background_color" name="' . $this->shared->get('slug') . '_select_background_color" class="regular-text" value="' . esc_attr(get_option("dagt_select_background_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the background color of the select element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function select_background_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'select_background_color', 'select_background_color', esc_attr__('Please enter a valid RGB color for the "Select Background Color" option.', 'dagt') );
            $output = get_option('dagt_select_background_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function select_border_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_select_border_color" name="' . $this->shared->get('slug') . '_select_border_color" class="regular-text" value="' . esc_attr(get_option("dagt_select_border_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the border color of the select element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function select_border_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'select_border_color', 'select_border_color', esc_attr__('Please enter a valid RGB color for the "Select Border Color" option.', 'dagt') );
            $output = get_option('dagt_select_border_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function select_arrow_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_select_arrow_color" name="' . $this->shared->get('slug') . '_select_arrow_color" class="regular-text" value="' . esc_attr(get_option("dagt_select_arrow_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the arrow color of the select element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function select_arrow_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'select_arrow_color', 'select_arrow_color', esc_attr__('Please enter a valid RGB color for the "Select Arrow Color" option.', 'dagt') );
            $output = get_option('dagt_select_arrow_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function select_highlight_background_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_select_highlight_background_color" name="' . $this->shared->get('slug') . '_select_highlight_background_color" class="regular-text" value="' . esc_attr(get_option("dagt_select_highlight_background_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option detemines the background color of the highlighted options in the select element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function select_highlight_background_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'select_highlight_background_color', 'select_highlight_background_color', esc_attr__('Please enter a valid RGB color for the "Select Highlight Background Color" option.', 'dagt') );
            $output = get_option('dagt_select_highlight_background_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function select_highlight_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_select_highlight_font_color" name="' . $this->shared->get('slug') . '_select_highlight_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_select_highlight_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the highlighted options in the select element.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function select_highlight_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'select_highlight_font_color', 'select_highlight_font_color', esc_attr__('Please enter a valid RGB color for the "Select Highlight Font Color" option.', 'dagt') );
            $output = get_option('dagt_select_highlight_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function tooltip_font_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_tooltip_font_color" name="' . $this->shared->get('slug') . '_tooltip_font_color" class="regular-text" value="' . esc_attr(get_option("dagt_tooltip_font_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the font color of the text available in the tooltip.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function tooltip_font_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'tooltip_font_color', 'tooltip_font_color', esc_attr__('Please enter a valid RGB color for the "Tooltip Font Color" option.', 'dagt') );
            $output = get_option('dagt_tooltip_font_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function tooltip_background_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_tooltip_background_color" name="' . $this->shared->get('slug') . '_tooltip_background_color" class="regular-text" value="' . esc_attr(get_option("dagt_tooltip_background_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the background color of the tooltip.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function tooltip_background_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'tooltip_background_color', 'tooltip_background_color', esc_attr__('Please enter a valid RGB color for the "Tooltip Background Color" option.', 'dagt') );
            $output = get_option('dagt_tooltip_background_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function tooltip_border_color_callback($args) {

        $html = '<input class="wp-color-picker" type="text" id="' . $this->shared->get('slug') . '_tooltip_border_color" name="' . $this->shared->get('slug') . '_tooltip_border_color" class="regular-text" value="' . esc_attr(get_option("dagt_tooltip_border_color")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the border color of the tooltip.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function tooltip_border_color_validation($input){

        if(!preg_match($this->shared->hex_rgb_regex, $input)){
            add_settings_error( 'tooltip_border_color', 'tooltip_border_color', esc_attr__('Please enter a valid RGB color for the "Tooltip Border Color" option.', 'dagt') );
            $output = get_option('dagt_tooltip_border_color');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    //labels options callbacks and validations ---------------------------------
    public function submit_button_text_callback($args){

        $html = '<input type="text" id="dagt_submit_button_text" name="dagt_submit_button_text" class="regular-text" value="' . esc_attr(get_option("dagt_submit_button_text")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the text displayed in the submit button.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function submit_button_text_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_submit_button_text', 'dagt_submit_button_text', esc_attr__('Please enter a valid text in the "Submit Button Text" option.', 'dagt') );
            $output = get_option('dagt_submit_button_text');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function reset_button_text_callback($args){

        $html = '<input type="text" id="dagt_reset_button_text" name="dagt_reset_button_text" class="regular-text" value="' . esc_attr(get_option("dagt_reset_button_text")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the text displayed in the reset button.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function reset_button_text_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_reset_button_text', 'dagt_reset_button_text', esc_attr__('Please enter a valid text in the "Submit Button Text" option.', 'dagt') );
            $output = get_option('dagt_reset_button_text');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function results_title_text_callback($args){

        $html = '<input type="text" id="dagt_results_title_text" name="dagt_results_title_text" class="regular-text" value="' . esc_attr(get_option("dagt_results_title_text")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the text displayed as title of the test results.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function results_title_text_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_results_title_text', 'dagt_results_title_text', esc_attr__('Please enter a valid text in the "Results Title Text" option.', 'dagt') );
            $output = get_option('dagt_results_title_text');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function evaluation_text_part_1_callback($args){

        $html = '<input type="text" id="dagt_evaluation_text_part_1" name="dagt_evaluation_text_part_1" class="regular-text" value="' . esc_attr(get_option("dagt_evaluation_text_part_1")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the first part of the evaluation text.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function evaluation_text_part_1_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_evaluation_text_part_1', 'dagt_evaluation_text_part_1', esc_attr__('Please enter a valid text in the "Evaluation Test Part 1" option.', 'dagt') );
            $output = get_option('dagt_evaluation_text_part_1');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function evaluation_text_part_2_callback($args){

        $html = '<input type="text" id="dagt_evaluation_text_part_2" name="dagt_evaluation_text_part_2" class="regular-text" value="' . esc_attr(get_option("dagt_evaluation_text_part_2")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the second part of the evaluation text.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function evaluation_text_part_2_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_evaluation_text_part_2', 'dagt_evaluation_text_part_2', esc_attr__('Please enter a valid text in the "Evaluation Test Part 2" option.', 'dagt') );
            $output = get_option('dagt_evaluation_text_part_2');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function evaluation_text_part_3_callback($args){

        $html = '<input type="text" id="dagt_evaluation_text_part_3" name="dagt_evaluation_text_part_3" class="regular-text" value="' . esc_attr(get_option("dagt_evaluation_text_part_3")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the third part of the evaluation text.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function evaluation_text_part_3_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_evaluation_text_part_3', 'dagt_evaluation_text_part_3', esc_attr__('Please enter a valid text in the "Evaluation Test Part 3" option.', 'dagt') );
            $output = get_option('dagt_evaluation_text_part_3');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function section_text_part_1_callback($args){

        $html = '<input type="text" id="dagt_section_text_part_1" name="dagt_section_text_part_1" class="regular-text" value="' . esc_attr(get_option("dagt_section_text_part_1")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the first part of the evaluation text displayed in the sections.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function section_text_part_1_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_section_text_part_1', 'dagt_section_text_part_1', esc_attr__('Please enter a valid text in the "Section Test Part 1" option.', 'dagt') );
            $output = get_option('dagt_section_text_part_1');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function section_text_part_2_callback($args){

        $html = '<input type="text" id="dagt_section_text_part_2" name="dagt_section_text_part_2" class="regular-text" value="' . esc_attr(get_option("dagt_section_text_part_2")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the second part of the evaluation text displayed in the sections.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function section_text_part_2_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_section_text_part_2', 'dagt_section_text_part_2', esc_attr__('Please enter a valid text in the "Section Test Part 2" option.', 'dagt') );
            $output = get_option('dagt_section_text_part_2');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function section_text_part_3_callback($args){

        $html = '<input type="text" id="dagt_section_text_part_3" name="dagt_section_text_part_3" class="regular-text" value="' . esc_attr(get_option("dagt_section_text_part_3")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the third part of the evaluation text displayed in the sections.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function section_text_part_3_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_section_text_part_3', 'dagt_section_text_part_3', esc_attr__('Please enter a valid text in the "Section Test Part 3" option.', 'dagt') );
            $output = get_option('dagt_section_text_part_3');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function non_public_test_text_callback($args){

        $html = '<input type="text" id="dagt_non_public_test_text" name="dagt_non_public_test_text" class="regular-text" value="' . esc_attr(get_option("dagt_non_public_test_text")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the text displayed to the users not authorized to complete non-public tests.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function non_public_test_text_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_non_public_test_text', 'dagt_non_public_test_text', esc_attr__('Please enter a valid text in the "Non-Public Test Text" option.', 'dagt') );
            $output = get_option('dagt_non_public_test_text');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function submitted_examination_text_callback($args){

        $html = '<input type="text" id="dagt_submitted_examination_text" name="dagt_submitted_examination_text" class="regular-text" value="' . esc_attr(get_option("dagt_submitted_examination_text")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the text displayed to the students that have completed an examination.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function submitted_examination_text_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_submitted_examination_text', 'dagt_submitted_examination_text', esc_attr__('Please enter a valid text in the "Submitted Examination Text" option.', 'dagt') );
            $output = get_option('dagt_submitted_examination_text');
        }else{
            $output = $input;
        }

        return $output;

    }

    //misc options callbacks and validations -----------------------------------
    public function random_input_offset_callback($args){

        $html = '<input type="text" id="dagt_random_input_offset" name="dagt_random_input_offset" class="regular-text" value="' . esc_attr(get_option("dagt_random_input_offset")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("The width of the input fields ( used for the missing words ) is by default determined by width of the correct answer. With the purpose of not giving hints to your users, this option allows you to increase the input fields width by a random number of characters, included between zero and the defined value of this option.", 'dagt')) . '"></div>';
        echo $html;

    }
    
    public function random_input_offset_validation($input){

        if( intval($input, 10) > 1000 ){
            add_settings_error( 'dagt_random_input_offset', 'dagt_random_input_offset', esc_attr__('Please enter a number from 0 to 1000 in the "Random Input Offset" option.', 'dagt') );
            $output = get_option('dagt_random_input_offset');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function test_scroll_offset_callback($args){

        $html = '<input type="text" id="dagt_test_scroll_offset" name="dagt_test_scroll_offset" class="regular-text" value="' . esc_attr(get_option("dagt_test_scroll_offset")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("With this option you can add or remove an offset to the destination of the vertical scroll executed when a test is submitted or reset.", 'dagt')) . '"></div>';
        echo $html;

    }
    
    public function test_scroll_offset_validation($input){

        if( intval($input, 10) > 10000 ){
            add_settings_error( 'dagt_test_scroll_offset', 'dagt_test_scroll_offset', esc_attr__('Please enter a number from 0 to 10000 in the "Test Scroll Offset" option.', 'dagt') );
            $output = get_option('dagt_test_scroll_offset');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function test_scroll_time_callback($args){

        $html = '<input type="text" id="dagt_test_scroll_time" name="dagt_test_scroll_time" class="regular-text" value="' . esc_attr(get_option("dagt_test_scroll_time")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the time in millisenconds required to complete the vertical scroll executed when a test is submitted or reset.", 'dagt')) . '"></div>';
        echo $html;

    }
    
    public function test_scroll_time_validation($input){

        if( intval($input, 10) > 10000 ){
            add_settings_error( 'dagt_test_scroll_time', 'dagt_test_scroll_time', esc_attr__('Please enter a number from 0 to 10000 in the "Test Scroll Time" option.', 'dagt') );
            $output = get_option('dagt_test_scroll_time');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function explanation_dashicon_callback($args){

        $html = '<input type="text" id="dagt_explanation_dashicon" name="dagt_explanation_dashicon" class="regular-text" value="' . esc_attr(get_option("dagt_explanation_dashicon")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__('This option determines the icon used to display the tooltips. You can find the complete list of the available icons by serching "WordPress Dashicons" on Google.', 'dagt')) . '"></div>';
        echo $html;

    }

    public function explanation_dashicon_validation($input){

        if(strlen(trim($input)) == 0){
            add_settings_error( 'dagt_explanation_dashicon', 'dagt_explanation_dashicon', esc_attr__('Please enter a valid text in the "Explanation Dashicon" option.', 'dagt') );
            $output = get_option('dagt_explanation_dashicon');
        }else{
            $output = $input;
        }

        return $output;

    }
    
    public function dashicon_size_callback($args){

        $html = '<input type="text" id="dagt_dashicon_size" name="dagt_dashicon_size" class="regular-text" value="' . esc_attr(get_option("dagt_dashicon_size")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__('This option determines the size of the icon used to display the tooltips.', 'dagt')) . '"></div>';
        
        echo $html;

    }

    public function dashicon_size_validation($input){

        if( intval($input, 10) < 1 or intval($input, 10) > 100 ){
            add_settings_error( 'dagt_dashicon_size', 'dagt_dashicon_size', esc_attr__('Please enter a number from 1 to 100 in the "Dashicon Size" option.', 'dagt') );
            $output = get_option('dagt_dashicon_size');
        }else{
            $output = $input;
        }

        return intval($output,  10);

    }
    
    public function load_google_font_1_callback($args){

        $html = '<input type="text" id="dagt_load_google_font_1" name="dagt_load_google_font_1" class="regular-text" value="' . esc_attr(get_option("dagt_load_google_font_1")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("Enter the URL of a Google Font ( with the proper parameters if needed ) to load the font on the front-end of your website.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function load_google_font_1_validation($input){
        
        if( strlen(trim($input)) != 0 and !preg_match($this->shared->url_regex, trim($input))){
            add_settings_error( 'dagt_load_google_font_1', 'dagt_load_google_font_1', esc_attr__('Please enter a valid URL in the "Load Google Font 1" option.', 'dagt') );
            $output = get_option('dagt_load_google_font_1');
        }else{
            $output = trim($input);
        }

        return $output;

    }
    
    public function load_google_font_2_callback($args){

        $html = '<input type="text" id="dagt_load_google_font_2" name="dagt_load_google_font_2" class="regular-text" value="' . esc_attr(get_option("dagt_load_google_font_2")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr(__("Enter the URL of a Google Font ( with the proper parameters if needed ) to load the font on the front-end of your website.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function load_google_font_2_validation($input){

        if( strlen(trim($input)) != 0 and !preg_match($this->shared->url_regex, trim($input))){
            add_settings_error( 'dagt_load_google_font_2', 'dagt_load_google_font_2', esc_attr__('Please enter a valid URL in the "Load Google Font 2" option.', 'dagt') );
            $output = get_option('dagt_load_google_font_2');
        }else{
            $output = trim($input);
        }

        return $output;

    }
    
    public function correct_answers_decoration_callback($args){

        $html = '<select class="daext-display-none" id="dagt_correct_answers_decoration" name="dagt_correct_answers_decoration">';
            $html .= '<option ' . selected(intval(get_option("dagt_correct_answers_decoration"), 10), 1, false) . ' value="1">' . esc_attr__('None', 'dagt') . '</option>';
            $html .= '<option ' . selected(intval(get_option("dagt_correct_answers_decoration"), 10), 2, false) . ' value="2">' . esc_attr__('Underline', 'dagt') . '</option>';
            $html .= '<option ' . selected(intval(get_option("dagt_correct_answers_decoration"), 10), 3, false) . ' value="3">' . esc_attr__('Line Through', 'dagt') . '</option>';
        $html .= '</select>';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the text decoration of the correct answers.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function correct_answers_decoration_validation($input){

        return $input;

    }
    
    public function wrong_answers_decoration_callback($args){

        $html = '<select class="daext-display-none" id="dagt_wrong_answers_decoration" name="dagt_wrong_answers_decoration">';
            $html .= '<option ' . selected(intval(get_option("dagt_wrong_answers_decoration"), 10), 1, false) . ' value="1">' . esc_attr__('None', 'dagt') . '</option>';
            $html .= '<option ' . selected(intval(get_option("dagt_wrong_answers_decoration"), 10), 2, false) . ' value="2">' . esc_attr__('Underline', 'dagt') . '</option>';
            $html .= '<option ' . selected(intval(get_option("dagt_wrong_answers_decoration"), 10), 3, false) . ' value="3">' . esc_attr__('Line Through', 'dagt') . '</option>';
        $html .= '</select>';
        $html .= '<div class="help-icon" title="' . esc_attr(__("This option determines the text decoration of the wrong answers.", 'dagt')) . '"></div>';
        echo $html;

    }

    public function wrong_answers_decoration_validation($input){

        return $input;

    }

    public function tests_menu_capability_callback($args){

        $html = '<input type="text" id="dagt_tests_menu_capability" name="dagt_tests_menu_capability" class="regular-text" value="' . esc_attr(get_option("dagt_tests_menu_capability")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr__('The capability required to get access on the "Tests" menu.', 'dagt') . '"></div>';
        echo $html;

    }

    public function tests_menu_capability_validation($input){

        if( strlen(trim($input)) != 0 and !preg_match($this->shared->regex_capability, trim($input))){
            add_settings_error( 'dagt_tests_menu_capability', 'dagt_tests_menu_capability', esc_attr__('Please enter a valid capability in the "Tests Menu Capability" option.', 'dagt') );
            $output = get_option('dagt_tests_menu_capability');
        }else{
            $output = trim($input);
        }

        return $output;

    }

    public function sections_menu_capability_callback($args){

        $html = '<input type="text" id="dagt_sections_menu_capability" name="dagt_sections_menu_capability" class="regular-text" value="' . esc_attr(get_option("dagt_sections_menu_capability")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr__('The capability required to get access on the "Sections" menu.', 'dagt') . '"></div>';
        echo $html;

    }

    public function sections_menu_capability_validation($input){

        if( strlen(trim($input)) != 0 and !preg_match($this->shared->regex_capability, trim($input))){
            add_settings_error( 'dagt_sections_menu_capability', 'dagt_sections_menu_capability', esc_attr__('Please enter a valid capability in the "Sections Menu Capability" option.', 'dagt') );
            $output = get_option('dagt_sections_menu_capability');
        }else{
            $output = trim($input);
        }

        return $output;

    }

    public function students_menu_capability_callback($args){

        $html = '<input type="text" id="dagt_students_menu_capability" name="dagt_students_menu_capability" class="regular-text" value="' . esc_attr(get_option("dagt_students_menu_capability")) . '" />';
        $html .= '<div class="help-icon" title="' . esc_attr__('The capability required to get access on the "Students" menu.', 'dagt') . '"></div>';
        echo $html;

    }

    public function students_menu_capability_validation($input){

        if( strlen(trim($input)) != 0 and !preg_match($this->shared->regex_capability, trim($input))){
            add_settings_error( 'dagt_students_menu_capability', 'dagt_students_menu_capability', esc_attr__('Please enter a valid capability in the "Students Menu Capability" option.', 'dagt') );
            $output = get_option('dagt_students_menu_capability');
        }else{
            $output = trim($input);
        }

        return $output;

    }

    /*
     * The "Export CSV" buttons and/or icons available in the Dashboard, Juice
     * and Hits menus are intercepted and the proper method that generates on
     * the fly the specific downloadable CSV file is called
     */
    public function export_csv_controller(){

        /*
         * Intercept requests that come from the "Export CSV" button from the
         * "Tests" menu and generate the downloadable CSV file with the
         * tests_menu_export_csv() method
         */
        if( isset($_GET['page']) and
            $_GET['page'] == 'dagt-tests' and
            isset($_POST['export_csv']) and
            isset($_POST['test_id']))
        {
            $this->tests_menu_export_csv();
        }

        /*
         * Intercept requests that come from the "Export CSV" button from the
         * "Students" menu and generate the downloadable CSV file with the
         * students_menu_export_csv() method
         */
        if( isset($_GET['page']) and
            $_GET['page'] == 'dagt-students' and
            isset($_POST['export_csv']) and
            isset($_POST['user_id']) )
        {
            $this->students_menu_export_csv();
        }

    }

    /*
     * Generates the downloadable CSV file with all the results of a specified
     * test
     */
    private function tests_menu_export_csv(){

        //verify capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_tests_menu_capability'))){die();}

        //get the test id
        $test_id = intval($_POST['test_id'], 10);

        //get the data from the db table
        global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE test_id = %d ORDER BY id DESC", $test_id);
        $results = $wpdb->get_results($safe_sql, ARRAY_A);

        //if there are data generate the csv header and content
        if( count( $results ) > 0 ){

            $csv_content = '';
            $new_line = "\n";

            //set the csv header
            header('Content-Encoding: UTF-8');
            header('Content-type: text/csv; charset=UTF-8');
            header("Content-Disposition: attachment; filename=test-" . $test_id . "-" . time() . ".csv");
            header("Pragma: no-cache");
            header("Expires: 0");

            //set headings
            $csv_content .= '"' . $this->esc_csv(__('Student', 'dagt')) . '",';
            $csv_content .= '"' . $this->esc_csv(__('Submission Date', 'dagt')) . '",';

            //set the titles of the 10 sections
            for($i=1;$i<=10;$i++){
                $csv_content .= '"S' . $i . '",';
            }

            $csv_content .= '"' . $this->esc_csv(__('Overall Score', 'dagt')) . '",';
            $csv_content .= '"' . $this->esc_csv(__('Percentage', 'dagt')) . '"';
            $csv_content .= $new_line;

            //set column content
            foreach ( $results as $result ) {

                //get the user id
                $user_obj = get_userdata($result['user_id']);
                $user_display_name = $user_obj->display_name;

                //calculate the overall score
                $total_score = $result['score_s1'] +
                               $result['score_s2'] +
                               $result['score_s3'] +
                               $result['score_s4'] +
                               $result['score_s5'] +
                               $result['score_s6'] +
                               $result['score_s7'] +
                               $result['score_s8'] +
                               $result['score_s9'] +
                               $result['score_s10'];

                $total_elements = $result['elements_s1'] +
                               $result['elements_s2'] +
                               $result['elements_s3'] +
                               $result['elements_s4'] +
                               $result['elements_s5'] +
                               $result['elements_s6'] +
                               $result['elements_s7'] +
                               $result['elements_s8'] +
                               $result['elements_s9'] +
                               $result['elements_s10'];

                $overall_score = $total_score . '/' . $total_elements;

                //calculate the percentage
                if($total_elements == 0){
                    $percentage = 0 . "%";
                }else{
                    $percentage = intval($total_score / $total_elements * 100, 10) . "%";
                }

                $csv_content .= '"' . $this->esc_csv($user_display_name) . '",';
                $csv_content .= '"' . $this->esc_csv( mysql2date( get_option('date_format') , $result['date'] ) ) . '",';

                //generate the data of the 10 section data
                for($i=1;$i<=10;$i++){
                    $csv_content .= '"' . $this->esc_csv($result['score_s' . $i] . '/' . $result['elements_s' . $i]) . '",';
                }

                $csv_content .= '"' . $this->esc_csv($overall_score) . '",';
                $csv_content .= '"' . $this->esc_csv($percentage) . '"';
                $csv_content .= $new_line;

            }

        }else{
            return false;
        }

        echo $csv_content;
        die();

    }

    /*
     * Generates the downloadable CSV file with all the items available in the
     * Students menu
     */
    private function students_menu_export_csv(){

        //verify capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_students_menu_capability'))){die();}

        //get the user id
        $user_id = intval($_POST['user_id'], 10);

        //get the user display name
        $user_obj = get_userdata($user_id);
        $user_display_name = $user_obj->display_name;

        //get the data from the db table
        global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE user_id = %d ORDER BY id DESC", $user_id);
        $results = $wpdb->get_results($safe_sql, ARRAY_A);

        //if there are data generate the csv header and content
        if( count( $results ) > 0 ){

            $csv_content = '';
            $new_line = "\n";

            //set the csv header
            header('Content-Encoding: UTF-8');
            header('Content-type: text/csv; charset=UTF-8');
            header("Content-Disposition: attachment; filename=user-" . $user_id . "-" . time() . ".csv");
            header("Pragma: no-cache");
            header("Expires: 0");

            //set headings
            $csv_content .= '"' . $this->esc_csv(__('Test Title', 'dagt')) . '",';
            $csv_content .= '"' . $this->esc_csv(__('Submission Date', 'dagt')) . '",';

            //get the max number of sections available in the results
            $section_counter = $this->shared->get_number_of_sections($results);

            //set the titles of the 10 sections
            for($i=1;$i<=10;$i++){
                $csv_content .= '"' . $this->esc_csv('S' . $i ) . '",';
            }

            $csv_content .= '"' . $this->esc_csv(__('Overall Score', 'dagt')) . '",';
            $csv_content .= '"' . $this->esc_csv(__('Percentage', 'dagt')) . '"';
            $csv_content .= $new_line;

            //set column content
            foreach ( $results as $result ) {

                //calculate the overall score
                $total_score = $result['score_s1'] +
                               $result['score_s2'] +
                               $result['score_s3'] +
                               $result['score_s4'] +
                               $result['score_s5'] +
                               $result['score_s6'] +
                               $result['score_s7'] +
                               $result['score_s8'] +
                               $result['score_s9'] +
                               $result['score_s10'];

                $total_elements = $result['elements_s1'] +
                               $result['elements_s2'] +
                               $result['elements_s3'] +
                               $result['elements_s4'] +
                               $result['elements_s5'] +
                               $result['elements_s6'] +
                               $result['elements_s7'] +
                               $result['elements_s8'] +
                               $result['elements_s9'] +
                               $result['elements_s10'];

                $overall_score = $total_score . '/' . $total_elements;

                //calculate the percentage
                if($total_elements == 0){
                    $percentage = 0 . "%";
                }else{
                    $percentage = intval($total_score / $total_elements * 100, 10) . "%";
                }

                $csv_content .= '"' . $this->esc_csv($this->shared->get_test_title($result['test_id'])) . '",';
                $csv_content .= '"' . $this->esc_csv( mysql2date( get_option('date_format') , $result['date'] ) ) . '",';

                //generate the data of the 10 section data
                for($i=1;$i<=10;$i++){
                    $csv_content .= '"' . $this->esc_csv( $result['score_s' . $i] . '/' . $result['elements_s' . $i]) . '",';
                }

                $csv_content .= '"' . $this->esc_csv($overall_score) . '",';
                $csv_content .= '"' . $this->esc_csv($percentage) . '"';
                $csv_content .= $new_line;

            }

        }else{
            return false;
        }

        echo $csv_content;
        die();

    }

    /*
     * Escape the double quotes of the $content string, so the returned string
     * can be used in CSV fields enclosed by double quotes
     *
     * @param $content The unescape content ( Ex: She said "No!" )
     * @return string The escaped content ( Ex: She said ""No!"" )
     */
    private function esc_csv($content){
        return str_replace('"', '""', $content);
    }

    /*
     * Given the user_id the overall number of correct answers is returned
     *
     * @param $user_id The user id
     * @return int The overall number of correct answers
     */
    public function get_correct_answers_of_student($user_id){

        /*
         * get the overall number of correct answers from the "result" db table
         */
        global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE user_id = %d ", $user_id);
        $results_a = $wpdb->get_results($safe_sql, ARRAY_A);

        $number_of_correct_answers = 0;

        foreach ($results_a as $key => $single_result) {

            $number_of_correct_answers = $number_of_correct_answers +
                                         $single_result['score_s1'] +
                                         $single_result['score_s2'] +
                                         $single_result['score_s3'] +
                                         $single_result['score_s4'] +
                                         $single_result['score_s5'] +
                                         $single_result['score_s6'] +
                                         $single_result['score_s7'] +
                                         $single_result['score_s8'] +
                                         $single_result['score_s9'] +
                                         $single_result['score_s10'];

        }

        return $number_of_correct_answers;

    }

    /*
     * Given the user_id the overall number of answers is returned
     *
     * @param $user_id The user id
     * @return int The overall number of answers
     */
    public function get_total_answers_of_student($user_id){

        /*
         * get the overall number of answers from the "result" db table
         */
        global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE user_id = %d ", $user_id);
        $results_a = $wpdb->get_results($safe_sql, ARRAY_A);

        $number_of_answers = 0;

        foreach ($results_a as $key => $single_result) {

            $number_of_answers = $number_of_answers +
                                 $single_result['elements_s1'] +
                                 $single_result['elements_s2'] +
                                 $single_result['elements_s3'] +
                                 $single_result['elements_s4'] +
                                 $single_result['elements_s5'] +
                                 $single_result['elements_s6'] +
                                 $single_result['elements_s7'] +
                                 $single_result['elements_s8'] +
                                 $single_result['elements_s9'] +
                                 $single_result['elements_s10'];

        }

        return $number_of_answers;

    }

    /**
     * Add the proper filters used to register the TinyMCE plugin and the TinyMCE button
     */
    function create_tinymce_plugin() {

       if ( !current_user_can(get_option( $this->shared->get('slug') . "_tests_menu_capability")) ) {
          return;
       }

       if ( get_user_option('rich_editing') == 'true' ) {

          //filter used to create the tinymce plugin
          add_filter( 'mce_external_plugins', array( $this, 'add_tinymce_plugin' ) );

          //filter used to create the tinymce button
          add_filter( 'mce_buttons', array( $this, 'register_tinymce_buttons' ) );

       }

    }

    //Add the 'grammartest' tinymce plugin
    public function add_tinymce_plugin( $plugin_array ) {

       $plugin_array['grammartest'] = $this->shared->get("url") . 'admin/assets/js/tinymce-grammartest.js';
       return $plugin_array;

    }

    /*
     * Add the "selectgrammartest" tinymce buttons
     *
     * @return array
     */
    public function register_tinymce_buttons( $buttons ) {

       array_push( $buttons, "", "selectgrammartest" );
       return $buttons;

    }
    
    /*
     * Delete the results associated with a specific user
     * 
     * @param $user_id The user id
     */
    public function delete_user_results($user_id){
        
        global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("DELETE FROM $table_name WHERE user_id = %d ", $user_id);
        $wpdb->query($safe_sql);
        
    }

}