<?php

//Prevent direct access to this file
if ( ! defined( 'WPINC' ) ) {
    die();
}

/**
 * Enqueue the Gutenberg block assets for the backend.
 *
 * 'wp-blocks': includes block type registration and related functions.
 * 'wp-element': includes the WordPress Element abstraction for describing the structure of your blocks.
 */
function dagt_editor_assets() {

    $shared = Dagt_Shared::get_instance();

    //Styles -----------------------------------------------------------------------------------------------------------

    //Block
    wp_enqueue_style(
        'dagt-editor-css',
        plugins_url( 'dist/editor.css', dirname( __FILE__ ) ),
        array( 'wp-edit-blocks' )//Dependency to include the CSS after it.
    );

    //Scripts ----------------------------------------------------------------------------------------------------------

    //Block
    wp_enqueue_script(
        'dagt-editor-js', // Handle.
        plugins_url( '/dist/blocks.build.js', dirname( __FILE__ ) ), //We register the block here.
        array( 'wp-blocks', 'wp-element' ), // Dependencies.
        false,
        true //Enqueue the script in the footer.
    );

    /*
     * Pass the objectL10n object to the javascript file with the 'dagt-editor-js' handle.
     */
    wp_localize_script( 'dagt-editor-js', 'dagtObjectL10n', array(
        'selectPlaceholder'  => esc_attr__('Select...', 'dagt'),
        'selectNoOptionsMessage' => esc_attr__('No options', 'dagt'),
        'grammarKeyword' => esc_attr__('grammar', 'dagt'),
        'testKeyword' => esc_attr__('test', 'dagt'),
        'examinationKeyword' => esc_attr__('examination', 'dagt')
    ) );

}
add_action( 'enqueue_block_editor_assets', 'dagt_editor_assets' );

/**
 * Enqueue the Gutenberg block assets for both frontend and backend.
 */
function dagt_style_assets() {

    //Not used with this block.
    return;

    //Styles -----------------------------------------------------------------------------------------------------------
    wp_enqueue_style(
        'dagt-style-css',
        plugins_url( 'dist/style.css', dirname( __FILE__ ) ),
        array( 'wp-blocks' )//Dependency to include the CSS after it
    );

}
add_action( 'enqueue_block_assets', 'dagt_style_assets' );

/**
 * Dynamic Block Server Component
 *
 * For more info:
 *
 * https://wordpress.org/gutenberg/handbook/blocks/creating-dynamic-blocks/
 */
function dagt_select_test_render($attributes) {

    if(isset($attributes['testId'])){
        $public = Dagt_Public::get_instance();
        return $public->display_grammar_test(array('id' => $attributes['testId']));
    }

}
register_block_type( 'dagt/select-test', array(
    'render_callback' => 'dagt_select_test_render',
) );