<?php

/*
 * this class should be used to include ajax actions
 */
class Dagt_Ajax{
    
    protected static $instance = null;
    private $shared = null;
    
    private function __construct() {
        
        //assign an instance of the plugin info
        $this->shared = Dagt_Shared::get_instance();
        
        //analyze the test
        add_action( 'wp_ajax_dagt_analyze_test', array( $this, 'analyze_test' ) );
        add_action( 'wp_ajax_nopriv_dagt_analyze_test', array( $this, 'analyze_test' ) );
        
        //get the test html
        add_action( 'wp_ajax_dagt_get_test_html', array( $this, 'get_test_html' ) );
        add_action( 'wp_ajax_nopriv_dagt_get_test_html', array( $this, 'get_test_html' ) );
        
        //generate the new missing word modal window content
        add_action( 'wp_ajax_dagt_generate_nmw_modal_window', array( $this, 'generate_nmw_modal_window' ) );
        
        //generate the edit missing word modal window content
        add_action( 'wp_ajax_dagt_generate_emw_modal_window', array( $this, 'generate_emw_modal_window' ) );
        
        //generate the new multiple selection modal window content
        add_action( 'wp_ajax_dagt_generate_nms_modal_window', array( $this, 'generate_nms_modal_window' ) );
        
        //generate the edit multiple selection modal window content
        add_action( 'wp_ajax_dagt_generate_ems_modal_window', array( $this, 'generate_ems_modal_window' ) );
        
        //save the data of a specific missing word
        add_action( 'wp_ajax_dagt_save_missing_word', array( $this, 'save_missing_word' ) );
        
        //update the data of a specific missing word
        add_action( 'wp_ajax_dagt_update_missing_word', array( $this, 'update_missing_word' ) );
        
        //save the data of a specific multiple selection
        add_action( 'wp_ajax_dagt_save_multiple_selection', array( $this, 'save_multiple_selection' ) );
        
        //update the data of a specific multiple selection
        add_action( 'wp_ajax_dagt_update_multiple_selection', array( $this, 'update_multiple_selection' ) );
        
        //update the missing word fields
        add_action( 'wp_ajax_dagt_update_missing_word_fields', array( $this, 'update_missing_word_fields' ) );
        
        //update the multiple selection fields
        add_action( 'wp_ajax_dagt_update_multiple_selection_fields', array( $this, 'update_multiple_selection_fields' ) );
        
        //get the number of valid missing word IDs in a provided array
        add_action( 'wp_ajax_dagt_get_number_of_valid_mw_ids', array( $this, 'get_number_of_valid_mw_ids' ) );
        
        //get the number of valid multiple selection IDs in a provided array
        add_action( 'wp_ajax_dagt_get_number_of_valid_ms_ids', array( $this, 'get_number_of_valid_ms_ids' ) );

        //get the modal window content for the TinyMCE modal window
        add_action('wp_ajax_dagt_get_modal_window_content', array( $this, 'get_modal_window_content'));

        //Generates a JSON string that includes name and ID of all the tests.
	    add_action( 'wp_ajax_dagt_get_test_list', array( $this, 'get_test_list' ) );
        
    }
    
    /*
     * return an istance of this class
     */
    public static function get_instance() {

        if ( null == self::$instance ) {
            self::$instance = new self;
        }

        return self::$instance;
            
    }

    /**
     * This method is used to retrieve the content of the modal window generated when the TinyMCE "League Table" button
     * is clicked.
     */
    function get_modal_window_content() {

        //check the referer
        if ( ! check_ajax_referer( 'dagt', 'security', false ) ) {
            echo "Invalid AJAX Request";
            die();
        }

        //check the capability
        if ( !current_user_can(get_option( $this->shared->get('slug') . '_tests_menu_capability')) )  {
            echo 'Invalid Capability';
            die();
        }

        $output = '<div id="dagt-grammartest-body-container">';
        $output .= '<h3>' . esc_attr__('Select the Test', 'dagt') . '</h3>';
        $output .= '<select id="test-id" data-placeholder="' . esc_attr__('Choose a Test', 'dagt') . '">';

        global $wpdb;
        $table_name = $wpdb->prefix . "dagt_test";
        $sql = "SELECT id, title FROM $table_name ORDER BY id DESC";
        $table_a = $wpdb->get_results($sql, ARRAY_A);
        foreach ($table_a as $key => $table) {
            $output .= '<option value="' . intval($table['id'], 10) .'">' . esc_attr(stripslashes($table['title'])) . '</option>';
        }

        $output .= '</select>';
        $output .= '</div>';

        echo $output;
        die();

    }
    
    /*
     * Analyze the test
     */
    public function analyze_test(){

        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //get the submitted test id
        $test_id = intval($_POST['test_id']);
        
        //get the data of the submitted test
        $test_data_a = $_POST['test_data'];
        
        if(count($test_data_a) == 0){echo 'No results';die();}
        
        //sanitize the data of the submitted test
        $sanitized_test_data_a = array();
        foreach($test_data_a as $key => $test_data){
            $sanitized_test_data_a[$key]['element_index'] = intval($test_data['element_index'], 10);
            $sanitized_test_data_a[$key]['section_id'] = intval($test_data['section_id'], 10);
            $sanitized_test_data_a[$key]['sentence_id'] = intval($test_data['sentence_id'], 10);
            $sanitized_test_data_a[$key]['type'] = addslashes($test_data['type']);
            $sanitized_test_data_a[$key]['type_id'] = intval($test_data['type_id'], 10);
            $sanitized_test_data_a[$key]['value'] = $test_data['value'];
        }
        
        //get the test data available in the "test" db table
        global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_test";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $test_id);
        $test_obj = $wpdb->get_row($safe_sql);
        
        //parse the 10 sections data of the submitted test
        for($i=1;$i<=10;$i++){
            
            //get the section id in the db table
            $section_id = $test_obj->{"section" . $i . "_id"};
            
            /*
             * If there is no section-identifier in the db table associated with
             * this section skip the rest of the for cycle
             */
            if($section_id == 0){continue;}
            
            //get the section data from the "section" db table
            $table_name = $wpdb->prefix . $this->shared->get('slug') . "_section";
            $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $section_id);
            $section_obj = $wpdb->get_row($safe_sql);
            
            /*
             * Compare the data of the 10 sentences available in the database
             * with the submitted data
             */
            for($t=1;$t<=10;$t++){
                
                $sentence = stripslashes($section_obj->{"sentence" . $t});

                //do not prooced if the sentence is empty
                if(strlen(trim($sentence)) == 0){continue;}
                
                /*
                 * Find the all the "mw" and the "ms" IDs with a regular
                 * expression
                 */
                preg_match_all(
                    '
                    {\[(m[ws])
                    \s+
                    id="
                    (\d+)
                    "\]
                    }ixu', # Enable case insentive, extended and unicode modifiers
                $sentence, $matches, PREG_OFFSET_CAPTURE);
                $captures = $matches[2];
                
                /*
                 * For each ID compare the correct value available in the
                 * database with the submitted data ( available in
                 * $sanitized_test_data_a )
                 */
                foreach($captures as $key => $single_capture){
                    
                    /*
                     * Verify if this is a "ms" or a "ms" by looking into the
                     * captured "shortcode" name
                     */
                    if($matches[1][$key][0] == 'mw'){
                        
                        //Missing Word -----------------------------------------
                        
                        /*
                         * Get the missing word data from the "missing_word" db
                         * table
                         */
                        $missing_word_id = $single_capture[0];
                        $table_name = $wpdb->prefix . $this->shared->get('slug') . "_missing_word";
                        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $missing_word_id);
                        $missing_word_obj = $wpdb->get_row($safe_sql);

                        /*
                         * If the Enable Multiple Correct Answers option is enabled populate $database_correct_answer
                         * by extracting the correct answers from the string.
                         *
                         * For example "must, should" produce the $database_correct_answer array with 'must' and 'should'.
                         */
                        if(intval(get_option( $this->shared->get('slug') . '_mw_multiple_correct_answer')) === 1){
	                        $database_correct_answer = explode(get_option( $this->shared->get('slug') . '_mw_multiple_correct_answer_separator'), trim($missing_word_obj->correct_answer));
                            $database_correct_answer = array_map('trim', $database_correct_answer);
                        }else{
                            $database_correct_answer = [trim($missing_word_obj->correct_answer)];
                        }

                        //Initialize the $correct_answer variable, used to display all the correct answers.
	                    $correct_answer = '';

                        /*
                         * Set the function used for the comparison based on the value of the case_insensitive field.
                         *
                         * Note that variable functions are used:
                         *
                         * http://php.net/manual/en/functions.variable-functions.php
                         */
	                    if($missing_word_obj->case_insensitive == 1) {
		                    $comparison_function = 'strcasecmp';
	                    }else{
		                    $comparison_function = 'strcmp';
	                    }

                        //set the section id
                        $section_id = $section_obj->id;

                        /*
                         * Set the sentence id ( that corresponds with the current
                         * value of the for cycle counter $t )
                         */
                        $sentence_id = $t;

                        /*
                         * Set the element_index ( that corresponds with the current
                         * value of the foreach $key
                         */
                        $element_index = $key;

                        /*
                         * Get the index of this specific array from the
                         * multidimensional array that stores the sanitized
                         * submitted data
                         */
                        $test_data_index = $this->shared->multidimensional_search($sanitized_test_data_a,
                            array(
                                'section_id'=>$section_id,
                                'sentence_id'=>$sentence_id,
                                'element_index'=>$element_index
                            )
                        );

                        /*
                         * Get the specific array that correponds to the retrieved
                         * index
                         */
                        $single_submitted_data = $sanitized_test_data_a[$test_data_index];

                        /*
                         * Get the value of the submitted answer
                         */
                        $submitted_answer = $single_submitted_data['value'];

                         //The $result flag, used to determine if the submitted answer is correct or not, is generated.
                        $result = 'wrong';
                        foreach($database_correct_answer as $key => $database_correct_answer_item){

                            //Compare the submitted answer with the correct answers defined in the back-end
                            if( $comparison_function($database_correct_answer_item, $submitted_answer) === 0 ){
                                $result = 'correct';
                            }

	                        //Populate the $correct_answer string, used to display all the correct answers
	                        if(strlen(trim($correct_answer)) === 0){
		                        $correct_answer .= trim($database_correct_answer_item);
	                        }else{
		                        $correct_answer .= get_option( $this->shared->get('slug') . '_mw_multiple_correct_answer_displayed_separator') . trim($database_correct_answer_item);
	                        }

                        }

                        /*
                         * Save the results of the analysis in the $results_a
                         * array
                         */
                        $results_a[] = array(
                            'section_id' => $single_submitted_data['section_id'],
                            'test_section_id' => $i,
                            'sentence_id' => $single_submitted_data['sentence_id'],
                            'element_index' => $single_submitted_data['element_index'],
                            'result' => $result,
                            'correct_answer' => esc_attr(stripslashes($correct_answer)),
                            'explanation' => esc_attr(stripslashes($missing_word_obj->explanation))
                        );
                        
                    }else{
                        
                        //Multiple Selection -----------------------------------
                        
                        /*
                         * Get the multiple selection data from the
                         * "multiple_selection" db table
                         */
                        $multiple_selection_id = $single_capture[0];
                        $table_name = $wpdb->prefix . $this->shared->get('slug') . "_multiple_selection";
                        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $multiple_selection_id);
                        $multiple_selection_obj = $wpdb->get_row($safe_sql);

                        //get the correct answer available in the database
                        $database_correct_answer = $multiple_selection_obj->correct_answer;

                        //set the section id
                        $section_id = $section_obj->id;

                        /*
                         * Set the sentence id ( that corresponds with the current
                         * value of the for cycle counter $t )
                         */
                        $sentence_id = $t;

                        /*
                         * Set the element_index
                         */
                        $element_index = $key;

                        /*
                         * Get the index of this specific array from the
                         * multidimensional array that stores the sanitized
                         * submitted data
                         */
                        $test_data_index = $this->shared->multidimensional_search($sanitized_test_data_a,
                            array(
                                'section_id'=>$section_id,
                                'sentence_id'=>$sentence_id,
                                'element_index'=>$element_index
                            )
                        );

                        /*
                         * Get the specific array that correponds to the retrieved
                         * index
                         */
                        $single_submitted_data = $sanitized_test_data_a[$test_data_index];

                        /*
                         * Get the value of the submitted answer
                         */
                        $submitted_answer = $single_submitted_data['value'];

                        //compare the database values with the submitted values
                        if($database_correct_answer == $submitted_answer){
                            $result = 'correct';
                        }else{
                            $result = 'wrong';
                        }

                        /*
                         * Save the results of the analysis in the $results_a
                         * array
                         */
                        $results_a[] = array(
                            'section_id' => $single_submitted_data['section_id'],
                            'test_section_id' => $i,
                            'sentence_id' => $single_submitted_data['sentence_id'],
                            'element_index' => $single_submitted_data['element_index'],
                            'result' => $result,
                            'correct_answer' => esc_attr(stripslashes($multiple_selection_obj->{"alternative_" . $database_correct_answer})),
                            'explanation' => esc_attr(stripslashes($multiple_selection_obj->explanation))
                        );
                        
                    }
                    
                }
                
            }
            
        }
        
        if(isset($results_a)){
            
            /*
             * If the test is an examination return the string
             * 'examination-completed'.
             * 
             * If the test is not an examination return the results of the test
             * as json. 
             */
            if(intval($test_obj->type, 10) == 4){
                $output = 'examination-completed';
            }else{
                $output = json_encode($results_a);   
            }
            
            
            //save the results in the "result" db table
            $this->shared->save_test_results($results_a, $test_id);
            
        }else{
            $output = 'No results';
        }
        
        echo $output;
        die();
        
    }
    
    /*
     * Generate the HTML code of the "Add Missing Word" modal window used in the
     * "Sections" menu
     */
    public function generate_nmw_modal_window(){

        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        ?>

        <div id="dagt-mw-modal-window-container">

            <div class="dagt-menu-head">
                <div class="dagt-menu-head-title"><?php esc_attr_e('Add Missing Word', 'dagt'); ?></div>
                <div class="dagt-menu-head-close"></div>
            </div>
            
            <div class="dagt-menu-content">
            
                <input type="hidden" id="nmw-missing-word-id" value="0">

                <h3><?php esc_attr_e('Correct Answer', 'dagt'); ?></h3>
                <input id="nmw-correct-answer" type="text" autocomplete="off">

                <h3><?php esc_attr_e('Case Insensitive', 'dagt'); ?></h3>
                <select id="nmw-case-insensitive">
                    <option value="1"><?php esc_attr_e('Yes', 'dagt'); ?></option>
                    <option value="0"><?php esc_attr_e('No', 'dagt'); ?></option>
                </select>

                <h3><?php esc_attr_e('Explanation', 'dagt'); ?></h3>
                <input id="nmw-explanation" type="text" autocomplete="off">
                
            </div>

            <div class="dagt-menu-footer">
                
                <a id="nmw-submit-button" class="submit-button"><?php esc_attr_e('Add Missing Word', 'dagt'); ?></a>
                
            </div>
            
        </div>  

        <?php
        
        die();
        
    }
    
    /*
     * Generate the HTML code of the "Edit Missing Word" modal window used in
     * the "Sections" menu
     */
    public function generate_emw_modal_window(){

        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        $sentence_content = stripslashes($_POST['sentence_content']);
        
        /*
         * Find a list of IDs of the missing word available in the sentence with
         * a regular expression.
         */
        preg_match_all(
            '
            {\[mw
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
        $sentence_content, $matches, PREG_OFFSET_CAPTURE);
        $captures = $matches[1];
        
        foreach($captures as $key => $single_capture){

            $missing_word_id_a[] = $single_capture[0];
        
        }
            
        ?>

        <div id="dagt-mw-modal-window-container">

            <div class="dagt-menu-head">
                <div class="dagt-menu-head-title"><?php esc_attr_e('Edit Missing Word', 'dagt'); ?></div>
                <div class="dagt-menu-head-close"></div>
            </div>
            
            <div class="dagt-menu-content">
            
                <h3><?php esc_attr_e('Missing Word ID', 'dagt'); ?></h3>
                <select id="emw-missing-word-id">
                    <?php
                    
                    $missing_word_counter = 0;
                    
                    //remove duplicates
                    $missing_word_id_a = array_unique($missing_word_id_a);

                    /*
                     * Get all the missing word available in this sentence from the
                     * "missing_word" db table
                     */
                    foreach($missing_word_id_a as $key => $missing_word_id){

                        //get from the database
                        global $wpdb;
                        $table_name = $wpdb->prefix . $this->shared->get('slug') . "_missing_word";
                        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $missing_word_id);
                        $missing_word_obj = $wpdb->get_row($safe_sql); 

                        if($missing_word_obj !== null){
                            
                            //output the option tag
                            echo '<option value="' . $missing_word_obj->id . '">' . $missing_word_obj->id . '</option>';

                            /*
                             * Save the data of the first missing word, will be later
                             * used to fill the input fields
                             */
                            if($missing_word_counter == 0){
                                $first_missing_word_obj = $missing_word_obj;
                            }
                            $missing_word_counter++;
                            
                        }
                        
                    }

                    ?>
                </select>

                <?php

                /*
                 * Use the data of the first missing word to fill the input fields
                 */

                ?>

                <!-- save the missing word id being edited -->

                <h3><?php esc_attr_e('Correct Answer', 'dagt'); ?></h3>
                <input id="emw-correct-answer" type="text" value="<?php echo esc_attr(stripslashes($first_missing_word_obj->correct_answer)); ?>" autocomplete="off">

                <h3><?php esc_attr_e('Case Insensitive', 'dagt'); ?></h3>
                <select id="emw-case-insensitive">
                    <option value="1" <?php selected(1, intval($first_missing_word_obj->case_insensitive, 10)); ?>><?php esc_attr_e('Yes', 'dagt'); ?></option>
                    <option value="0" <?php selected(0, intval($first_missing_word_obj->case_insensitive, 10)); ?>><?php esc_attr_e('No', 'dagt'); ?></option>
                </select>

                <h3><?php esc_attr_e('Explanation', 'dagt'); ?></h3>
                <input id="emw-explanation" type="text" value="<?php echo esc_attr(stripslashes($first_missing_word_obj->explanation)); ?>" autocomplete="off">
               
            </div>

            <div class="dagt-menu-footer">
            
                <a id="emw-submit-button" class="submit-button"><?php esc_attr_e('Update Missing Word', 'dagt'); ?></a>

            </div>
            
        </div>  

        <?php
        
        die();
        
    }
    
    /*
     * Generate the HTML code of the "Add Multiple Selection" modal window used
     * in the "Sections" menu
     */
    public function generate_nms_modal_window(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        ?>
        
        <div id="dagt-mw-modal-window-container">

            <div class="dagt-menu-head">
                <div class="dagt-menu-head-title"><?php esc_attr_e('Add Multiple Selection', 'dagt'); ?></div>
                <div class="dagt-menu-head-close"></div>
            </div>
            
            <div class="dagt-menu-content">
            
                <input type="hidden" id="nms-multiple-selection-id" value="0">

                <h3><?php esc_attr_e('Alternative 1', 'dagt'); ?></h3>
                <input id="nms-alternative-1" type="text" autocomplete="off">

                <h3><?php esc_attr_e('Alternative 2', 'dagt'); ?></h3>
                <input id="nms-alternative-2" type="text" autocomplete="off">

                <h3><?php esc_attr_e('Alternative 3', 'dagt'); ?></h3>
                <input id="nms-alternative-3" type="text" autocomplete="off">

                <h3><?php esc_attr_e('Alternative 4', 'dagt'); ?></h3>
                <input id="nms-alternative-4" type="text" autocomplete="off">

                <h3><?php esc_attr_e('Alternative 5', 'dagt'); ?></h3>
                <input id="nms-alternative-5" type="text" autocomplete="off">

                <h3><?php esc_attr_e('Correct Answer', 'dagt'); ?></h3>
                <select id="nms-correct-answer">
                    <option value="1"><?php esc_attr_e('Alternative 1', 'dagt'); ?></option>
                    <option value="2"><?php esc_attr_e('Alternative 2', 'dagt'); ?></option>
                    <option value="3"><?php esc_attr_e('Alternative 3', 'dagt'); ?></option>
                    <option value="4"><?php esc_attr_e('Alternative 4', 'dagt'); ?></option>
                    <option value="5"><?php esc_attr_e('Alternative 5', 'dagt'); ?></option>
                </select>

                <h3><?php esc_attr_e('Explanation', 'dagt'); ?></h3>
                <input id="nms-explanation" type="text" autocomplete="off">
                
            </div>

            <div class="dagt-menu-footer">
            
                <a id="nms-submit-button" class="submit-button"><?php esc_attr_e('Add Multiple Selection', 'dagt'); ?></a>
                
            </div>

        </div>  

        <?php
        
        die();
        
    }
    
    /*
     * Generate the HTML code of the "Edit Multiple Selection" modal window used
     * in the "Sections" menu
     */
    public function generate_ems_modal_window(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        $sentence_content = stripslashes($_POST['sentence_content']);
        
        /*
         * Find a list of IDs of the missing word available in the sentence with
         * a regular expression.
         */
        preg_match_all(
            '
            {\[ms
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
        $sentence_content, $matches, PREG_OFFSET_CAPTURE);
        $captures = $matches[1];
        
        foreach($captures as $key => $single_capture){

            $multiple_selection_id_a[] = $single_capture[0];
        
        }
            
        ?>
        
        <div id="dagt-mw-modal-window-container">
            
            <div class="dagt-menu-head">
                <div class="dagt-menu-head-title"><?php esc_attr_e('Edit Multiple Selection', 'dagt'); ?></div>
                <div class="dagt-menu-head-close"></div>
            </div>
            
            <div class="dagt-menu-content">
            
                <h3><?php esc_attr_e('Multiple Selection ID', 'dagt'); ?></h3>
                <select id="ems-multiple-selection-id">
                    <?php

                    $multiple_selection_counter = 0;
                    
                    //remove duplicates
                    $multiple_selection_id_a = array_unique($multiple_selection_id_a);

                    /*
                     * Get all the missing word available in this sentence from the
                     * "missing_word" db table
                     */
                    foreach($multiple_selection_id_a as $key => $multiple_selection_id){

                        //get from the database
                        global $wpdb;
                        $table_name = $wpdb->prefix . $this->shared->get('slug') . "_multiple_selection";
                        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $multiple_selection_id);
                        $multiple_selection_obj = $wpdb->get_row($safe_sql); 

                        if($multiple_selection_obj !== null){
                        
                            //output the option tag
                            echo '<option value="' . $multiple_selection_obj->id . '">' . $multiple_selection_obj->id . '</option>';

                            /*
                             * Save the data of the first missing word, will be later
                             * used to fill the input fields
                             */
                            if($multiple_selection_counter == 0){
                                $first_multiple_selection_obj = $multiple_selection_obj;
                            }
                            $multiple_selection_counter++;
                            
                        }

                    }

                    ?>
                </select>

                <h3><?php esc_attr_e('Alternative 1', 'dagt'); ?></h3>
                <input id="ems-alternative-1" type="text" value="<?php echo esc_attr(stripslashes($first_multiple_selection_obj->alternative_1)); ?>" autocomplete="off">

                <h3><?php esc_attr_e('Alternative 2', 'dagt'); ?></h3>
                <input id="ems-alternative-2" type="text" value="<?php echo esc_attr(stripslashes($first_multiple_selection_obj->alternative_2)); ?>" autocomplete="off">

                <h3><?php esc_attr_e('Alternative 3', 'dagt'); ?></h3>
                <input id="ems-alternative-3" type="text" value="<?php echo esc_attr(stripslashes($first_multiple_selection_obj->alternative_3)); ?>" autocomplete="off">

                <h3><?php esc_attr_e('Alternative 4', 'dagt'); ?></h3>
                <input id="ems-alternative-4" type="text" value="<?php echo esc_attr(stripslashes($first_multiple_selection_obj->alternative_4)); ?>" autocomplete="off">

                <h3><?php esc_attr_e('Alternative 5', 'dagt'); ?></h3>
                <input id="ems-alternative-5" type="text" value="<?php echo esc_attr(stripslashes($first_multiple_selection_obj->alternative_5)); ?>" autocomplete="off">

                <h3><?php esc_attr_e('Correct Answer', 'dagt'); ?></h3>
                <select id="ems-correct-answer">
                    <option value="1" <?php selected(1, intval($first_multiple_selection_obj->correct_answer, 10)); ?>><?php esc_attr_e('Alternative 1', 'dagt'); ?></option>
                    <option value="2" <?php selected(2, intval($first_multiple_selection_obj->correct_answer, 10)); ?>><?php esc_attr_e('Alternative 2', 'dagt'); ?></option>
                    <option value="3" <?php selected(3, intval($first_multiple_selection_obj->correct_answer, 10)); ?>><?php esc_attr_e('Alternative 3', 'dagt'); ?></option>
                    <option value="4" <?php selected(4, intval($first_multiple_selection_obj->correct_answer, 10)); ?>><?php esc_attr_e('Alternative 4', 'dagt'); ?></option>
                    <option value="5" <?php selected(5, intval($first_multiple_selection_obj->correct_answer, 10)); ?>><?php esc_attr_e('Alternative 5', 'dagt'); ?></option>
                </select>

                <h3><?php esc_attr_e('Explanation', 'dagt'); ?></h3>
                <input id="ems-explanation" type="text" value="<?php echo esc_attr(stripslashes($first_multiple_selection_obj->explanation)); ?>" autocomplete="off">
                
            </div>

            <div class="dagt-menu-footer">
            
                <a id="ems-submit-button" class="submit-button"><?php esc_attr_e('Update Multiple Selection', 'dagt'); ?></a>
                
            </div>

        </div>  

        <?php
        
        die();
        
    }
    
    /*
     * Save a missing word added with the "Add Missing Word" modal window in the
     * "Sections" menu
     */
    public function save_missing_word(){

        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        //get data from $_POST
        $missing_word_id = intval($_POST['missing_word_id']);
        $correct_answer = $_POST['correct_answer'];
        $case_insensitive = intval($_POST['case_insensitive']);
        $explanation = $_POST['explanation'];
            
        //insert the new missing word in the "missing_word" db table
        global $wpdb;
        $table_name = $wpdb->prefix . $this->shared->get('slug') . "_missing_word";
        $safe_sql = $wpdb->prepare("INSERT INTO $table_name SET 
            correct_answer = %s,
            explanation = %s,
            case_insensitive = %d",
            $correct_answer,
            $explanation,
            $case_insensitive
        );

        $query_result = $wpdb->query( $safe_sql );

        if($query_result !== false){
            
            /*
             * Output the id of the new missing word or the id of the updated
             * missing word
             */
            echo $wpdb->insert_id;
            
        }else{
            echo 'error';
        }
        
        die();
        
    }
    
    /*
     * Update a missing word changed with the "Edit Missing Word" modal window
     * in the "Sections" menu
     */
    public function update_missing_word(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        //get data from $_POST
        $missing_word_id = intval($_POST['missing_word_id']);
        $correct_answer = $_POST['correct_answer'];
        $case_insensitive = intval($_POST['case_insensitive']);
        $explanation = $_POST['explanation'];
        
        //update the missing word in the "missing_word" db table
        global $wpdb;
        $table_name = $wpdb->prefix . $this->shared->get('slug') . "_missing_word";
        $safe_sql = $wpdb->prepare("UPDATE $table_name SET 
            correct_answer = %s,
            explanation = %s,
            case_insensitive = %d
            WHERE id = %d",
            $correct_answer,
            $explanation,
            $case_insensitive,
            $missing_word_id
        );
        
        $query_result = $wpdb->query( $safe_sql );

        if($query_result !== false){
            
            /*
             * Output the id of the new missing word or the id of the updated
             * missing word
             */
            echo $wpdb->insert_id;
            
        }else{
            echo 'error';
        }
        
        die();
        
    }
    
    /*
     * Save a multiple selection added with the "Add Multiple Selection" modal
     * window in the "Sections" menu
     */
    public function save_multiple_selection(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        //get data from $_POST
        $multiple_selection_id = intval($_POST['multiple_selection_id']);
        $alternative_1 = $_POST['alternative_1'];
        $alternative_2 = $_POST['alternative_2'];
        $alternative_3 = $_POST['alternative_3'];
        $alternative_4 = $_POST['alternative_4'];
        $alternative_5 = $_POST['alternative_5'];
        $correct_answer = $_POST['correct_answer'];
        $explanation = $_POST['explanation'];
            
        //insert the new missing word in the "missing_word" db table
        global $wpdb;
        $table_name = $wpdb->prefix . $this->shared->get('slug') . "_multiple_selection";
        $safe_sql = $wpdb->prepare("INSERT INTO $table_name SET 
            alternative_1 = %s,
            alternative_2 = %s,
            alternative_3 = %s,
            alternative_4 = %s,
            alternative_5 = %s,
            correct_answer = %d,
            explanation = %s",
            $alternative_1,
            $alternative_2,
            $alternative_3,
            $alternative_4,
            $alternative_5,
            $correct_answer,
            $explanation
        );
        
        $query_result = $wpdb->query( $safe_sql );

        if($query_result !== false){
            
            /*
             * Output the id of the new missing word or the id of the updated
             * missing word
             */
            echo $wpdb->insert_id;
            
        }else{
            echo 'error';
        }
        
        die();
        
    }
    
    /*
     * Update a multiple selection changed with the "Edit Multiple Selection"
     * modal window in the "Sections" menu
     */
    public function update_multiple_selection(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        //get data from $_POST
        $multiple_selection_id = intval($_POST['multiple_selection_id']);
        $alternative_1 = $_POST['alternative_1'];
        $alternative_2 = $_POST['alternative_2'];
        $alternative_3 = $_POST['alternative_3'];
        $alternative_4 = $_POST['alternative_4'];
        $alternative_5 = $_POST['alternative_5'];
        $correct_answer = $_POST['correct_answer'];
        $explanation = $_POST['explanation'];
            
        //insert the new missing word in the "missing_word" db table
        global $wpdb;
        $table_name = $wpdb->prefix . $this->shared->get('slug') . "_multiple_selection";
        $safe_sql = $wpdb->prepare("UPDATE $table_name SET 
            alternative_1 = %s,
            alternative_2 = %s,
            alternative_3 = %s,
            alternative_4 = %s,
            alternative_5 = %s,
            correct_answer = %d,
            explanation = %s
            WHERE id = %d",
            $alternative_1,
            $alternative_2,
            $alternative_3,
            $alternative_4,
            $alternative_5,
            $correct_answer,
            $explanation,
            $multiple_selection_id
        );
        
        $query_result = $wpdb->query( $safe_sql );

        if($query_result !== false){
            
            /*
             * Output the id of the new missing word or the id of the updated
             * missing word
             */
            echo 'success';
            
        }else{
            echo 'error';
        }
        
        die();
        
    }
    
    /*
     * return the values of the missing word fields so that will be updated by
     * the javascript part
     */
    public function update_missing_word_fields(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        //get the missing word id
        $missing_word_id = intval($_POST['missing_word_id'], 10);
        
        //get the missing word data of the specific row from the db table
        global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_missing_word";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $missing_word_id);
        $missing_word_obj = $wpdb->get_row($safe_sql);
        
        //remove the slashes before sending the json response
        $missing_word_obj->correct_answer = stripslashes($missing_word_obj->correct_answer);
        $missing_word_obj->explanation = stripslashes($missing_word_obj->explanation);
        
        //return the data with json
        echo json_encode($missing_word_obj);
        
        die();
        
    }
    
    /*
     * Return the values of the multiple selection fields so that will be
     * updated by the javascript part
     */
    public function update_multiple_selection_fields(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        //get the missing word id
        $multiple_selection_id = intval($_POST['multiple_selection_id'], 10);
        
        //get the multiple selection data of the specific row from the db table
        global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_multiple_selection";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $multiple_selection_id);
        $multiple_selection_obj = $wpdb->get_row($safe_sql);
        
        //remove the slashes before sending the json response
        $multiple_selection_obj->alternative_1 = stripslashes($multiple_selection_obj->alternative_1);
        $multiple_selection_obj->alternative_2 = stripslashes($multiple_selection_obj->alternative_2);
        $multiple_selection_obj->alternative_3 = stripslashes($multiple_selection_obj->alternative_3);
        $multiple_selection_obj->alternative_4 = stripslashes($multiple_selection_obj->alternative_4);
        $multiple_selection_obj->alternative_5 = stripslashes($multiple_selection_obj->alternative_5);
        $multiple_selection_obj->explanation = stripslashes($multiple_selection_obj->explanation);
        
        //return the data with json
        echo json_encode($multiple_selection_obj);
        
        die();
        
    }
    
    /*
     * Get the test html by making use of the provided test_id
     */
    public function get_test_html(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //get the test id
        $test_id = intval($_POST['test_id'], 10);
        
        //generate the html of the specific test
        $test_html = $this->shared->generate_grammar_test_html($test_id);
        
        echo $test_html;
        die();
        
    }
    
    /*
     * Get the number of valid missing word IDs in a provided array
     */
    public function get_number_of_valid_mw_ids(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        //if no ids are sent return 0 as the number of valid ids
        if(!isset($_POST['mw_id_a'])){return 0;die();}
        
        $mw_id_a = $_POST['mw_id_a'];
        $number_of_valid_ids = 0;
        
        foreach($mw_id_a as $key => $mw_id){
            
            //verify if the id exists in the missing_word db table
            global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_missing_word";
            $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $mw_id);
            $wpdb->get_results($safe_sql);
            
            if($wpdb->num_rows > 0){
                $number_of_valid_ids++;
            }
            
        }
        
        echo $number_of_valid_ids;
        die();
        
    }
    
    /*
     * Get the number of valid multiple selection IDs in a provided array
     */
    public function get_number_of_valid_ms_ids(){
        
        //check the referer
        check_ajax_referer('dagt', 'security');
        
        //check the capability
        if(!current_user_can(get_option( $this->shared->get('slug') . '_sections_menu_capability'))){die();}
        
        //if no ids are sent return 0 as the number of valid ids
        if(!isset($_POST['ms_id_a'])){return 0;die();}
        
        $ms_id_a = $_POST['ms_id_a'];
        $number_of_valid_ids = 0;
        
        foreach($ms_id_a as $key => $ms_id){
            
            //verify if the id exists in the missing_word db table
            global $wpdb; $table_name = $wpdb->prefix . $this->shared->get('slug') . "_multiple_selection";
            $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $ms_id);
            $wpdb->get_results($safe_sql);
            
            if($wpdb->num_rows > 0){
                $number_of_valid_ids++;
            }
            
        }
        
        echo $number_of_valid_ids;
        die();
        
    }

	/**
	 * A JSON string that includes name and ID of all the tests is returned.
	 */
	function get_test_list() {

		//check the referer
		if ( ! check_ajax_referer( 'dagt', 'security', false ) ) {
			echo "Invalid AJAX Request";
			die();
		}

		global $wpdb;
		$table_name = $wpdb->prefix . "dagt_test";
		$sql = "SELECT id, title FROM $table_name ORDER BY id DESC";
		$test_a = $wpdb->get_results($sql, ARRAY_A);

		echo json_encode($test_a);
		die();

	}
    
}