<?php

/*
 * this class should be used to work with the public side of wordpress
 */
class Dagt_Public{
    
    //general class properties
    protected static $instance = null;
    private $shared = null;
    
    //Store all the shortcode IDs used in the post/page
    private static $shortcode_id_a = array();
    
    private $element_index = 0;
    
    private function __construct() {
        
        //assign an instance of the plugin info
        $this->shared = Dagt_Shared::get_instance();
        
        //write in front-end head
        add_action('wp_head', array( $this, 'wr_public_head') );
        
        //write hidden fields with data with the footer hook
        add_action('get_footer', array( $this, 'wr_hidden_fields_data') );
        
        //load public stylesheets
        add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_styles' ) );
        
        //load public js
        add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );
        
        //grammar-test shortcode
        add_shortcode('grammar-test', array( $this, 'display_grammar_test') );
        
    }
    
    /*
     * create an instance of this class
     */
    public static function get_instance() {

        if ( null == self::$instance ) {
            self::$instance = new self;
        }

        return self::$instance;
        
    }
    
    public function wr_public_head(){
        
        //javascript variables
        echo '<script type="text/javascript">';
            echo 'var dagt_ajax_url = "' . admin_url('admin-ajax.php') . '";';
            echo 'var dagt_nonce = "' . wp_create_nonce( "dagt" ) . '";';
        echo '</script>';
        
        //load google font 1 based on the plugin options
        if( strlen(trim(get_option( $this->shared->get('slug') . "_load_google_font_1"))) > 0 ){
             echo "<link href='" . esc_attr(get_option( $this->shared->get('slug') . "_load_google_font_1")) . "' rel='stylesheet' type='text/css'>";
        }

        //load google font 2 based on the plugin options
        if( strlen(trim(get_option($this->shared->get('slug') . "_load_google_font_2"))) > 0 ){
            echo "<link href='" . esc_attr(get_option( $this->shared->get('slug') . "_load_google_font_2")) . "' rel='stylesheet' type='text/css'>";
        }
        
    }
    
    /*
     * Write hidden field data with the footer hook
     */
    public function wr_hidden_fields_data(){
        
        //hidden fields used by the javascript part
        $output = '<input id="dagt-results-title-text" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_results_title_text')) . '">';
        $output .= '<input id="dagt-evaluation-text-part-1" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_evaluation_text_part_1')) . '">';
        $output .= '<input id="dagt-evaluation-text-part-2" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_evaluation_text_part_2')) . '">';
        $output .= '<input id="dagt-evaluation-text-part-3" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_evaluation_text_part_3')) . '">';
        $output .= '<input id="dagt-section-text-part-1" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_section_text_part_1')) . '">';
        $output .= '<input id="dagt-section-text-part-2" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_section_text_part_2')) . '">';
        $output .= '<input id="dagt-section-text-part-3" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_section_text_part_3')) . '">';
        $output .= '<input id="dagt-submitted-examination-text" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_submitted_examination_text')) . '">';
        $output .= '<input id="dagt-test-scroll-offset" type="hidden" value="' . intval(get_option($this->shared->get('slug') . '_test_scroll_offset'), 10) . '">';
        $output .= '<input id="dagt-test-scroll-time" type="hidden" value="' . intval(get_option($this->shared->get('slug') . '_test_scroll_time'), 10) . '">';
        $output .= '<input id="dagt-explanation-dashicon" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_explanation_dashicon')) . '">';
        $output .= '<input id="dagt-show-test-title" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_show_test_title')) . '">';
        $output .= '<input id="dagt-remove-parentheses" type="hidden" value="' . intval(get_option($this->shared->get('slug') . '_remove_parentheses'), 10) . '">';
        $output .= '<input id="dagt-auto-scroll" type="hidden" value="' . intval(get_option($this->shared->get('slug') . '_auto_scroll'), 10) . '">';
        $output .= '<input id="dagt-scroll-easing" type="hidden" value="' . esc_attr(get_option($this->shared->get('slug') . '_scroll_easing')) . '">';
        
        echo $output;
        
    }
    
    public function enqueue_styles() {
        wp_enqueue_style( $this->shared->get('slug') .'-general', $this->shared->get('url') . 'public/assets/css/general.css', array('dashicons'), $this->shared->get('ver') );
        wp_enqueue_style( $this->shared->get('slug') .'-custom', $this->shared->get('url') . 'public/assets/css/custom-' . get_current_blog_id() . '.css', array('dashicons'), $this->shared->get('ver') );
        wp_enqueue_style( $this->shared->get('slug') .'-jquery-ui-tooltip', $this->shared->get('url') . 'public/assets/css/jquery-ui-tooltip.css', array(), $this->shared->get('ver') );
        wp_enqueue_style( $this->shared->get('slug') .'-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.css', array(), $this->shared->get('ver') );
    }
    
    public function enqueue_scripts() {
        
        wp_enqueue_script( $this->shared->get('slug') . '-chosen', $this->shared->get('url') . 'shared/assets/inc/chosen/chosen.js', 'jquery', $this->shared->get('ver'), true );
        wp_enqueue_script( $this->shared->get('slug') . '-general', $this->shared->get('url') . 'public/assets/js/general.js', 'jquery', $this->shared->get('ver'), true );
        wp_enqueue_script( $this->shared->get('slug') . '-animatescroll', $this->shared->get('url') . 'public/assets/js/animatescroll.js', 'jquery', $this->shared->get('ver'), true );
        wp_enqueue_script('jquery-ui-tooltip');
        
    }
    
    /*
     * Callback for the [grammar-test] shortcode
     */
    public function display_grammar_test( $atts ){

        if( !is_feed() and ( is_single() or is_page() ) ){

            //get the event id
            extract( shortcode_atts( array('id' => '0'), $atts ) );
            $test_id = intval( $id, 10 );
            
            //verify if this grammar test id has already been used
            if(in_array($test_id, self::$shortcode_id_a, true)){
                return esc_attr__("You can't include multiple times the same test.", 'dagt');
            }
            
            //store the shortcode id
            self::$shortcode_id_a[] = $test_id;

            //generate the grammar test html
            $output = $this->shared->generate_grammar_test_html($test_id);
            
            return $output;

        }

    }
    
}