<?php

/*
 * this class should be used to stores properties and methods shared by the
 * admin and public side of wordpress
 */
class Dagt_Shared
{
    
    //regexp
    public $hex_rgb_regex = '/^#(?:[0-9a-fA-F]{3}){1,2}$/';
    public $font_family_regex = '/^([A-Za-z0-9-\'", ]*)$/';
    public $url_regex = '/^(http|https):\/\/[-A-Za-z0-9+&@#\/%?=~_|$!:,.;]+$/';
    public $regex_capability = '/^\s*[A-Za-z0-9_]+\s*$/';
    
    protected static $instance = null;

    private $data = array();

    private function __construct(){

        //Set plugin textdomain
        load_plugin_textdomain('dagt', false, 'grammar-test/lang/');
        
        $this->data['slug'] = 'dagt';
        $this->data['ver'] = '1.18';
        $this->data['dir'] = substr(plugin_dir_path(__FILE__), 0, -7);
        $this->data['url'] = substr(plugin_dir_url(__FILE__), 0, -7);

    }

    public static function get_instance(){

        if (null == self::$instance) {
            self::$instance = new self;
        }

        return self::$instance;

    }

    //retrieve data
    public function get($index){
        return $this->data[$index];
    }

    
    public function multidimensional_search($parents, $searched) { 
        
        if (empty($searched) || empty($parents)) { 
          return false; 
        } 

        foreach ($parents as $key => $value) { 
          $exists = true; 
          foreach ($searched as $skey => $svalue) { 
            $exists = ($exists && IsSet($parents[$key][$skey]) && $parents[$key][$skey] == $svalue); 
          } 
          if($exists){ return $key; } 
        } 

        return false; 
      
    }
    
    /*
     * Save the results of a test in the "result" db table.
     * 
     * @param $results_a Array which includes the results
     * @param $test_id The id of the test
     */
    public function save_test_results($results_a, $test_id = 0){
        
        /*
         * Save the test results only if the 'type' of the test is:
         * 
         * - practice (3)
         * - examination (4)
         */
        $test_type = $this->get_type_of_test($test_id);
        if( $test_type == 1 or $test_type == 2 or !current_user_can('dagt_perform_practice_and_examination') ){return;}
        
        //get the user id
        $user_id = get_current_user_id();
        
        //init fields
        $score_s1 = 0;
        $score_s2 = 0;
        $score_s3 = 0;
        $score_s4 = 0;
        $score_s5 = 0;
        $score_s6 = 0;
        $score_s7 = 0;
        $score_s8 = 0;
        $score_s9 = 0;
        $score_s10 = 0;
        $elements_s1 = 0;
        $elements_s2 = 0;
        $elements_s3 = 0;
        $elements_s4 = 0;
        $elements_s5 = 0;
        $elements_s6 = 0;
        $elements_s7 = 0;
        $elements_s8 = 0;
        $elements_s9 = 0;
        $elements_s10 = 0;
        
        //find the values
        foreach($results_a as $key => $result){
            
            //count the total elements for each section
            ${"elements_s" . $result['test_section_id']} = ${"elements_s" . $result['test_section_id']} + 1;
            
            //count the number of correct answers
            if($result['result'] == 'correct'){
                ${"score_s" . $result['test_section_id']} = ${"score_s" . $result['test_section_id']} + 1;
            }
            
        }
        
        //set the examination flag
        global $wpdb;
        $table_name = $wpdb->prefix . $this->get('slug') . "_test";
        $safe_sql = $wpdb->prepare("SELECT type FROM $table_name WHERE id = %d ", $test_id);
        $test_obj = $wpdb->get_row($safe_sql); 
        
        if(intval($test_obj->type, 10) == 4){
            $examination = true;
        }else{
            $examination = false;
        }
        
        //get the current date
        $date = current_time('mysql');
        $date_gmt = current_time('mysql', 1);
        
        /*
         * salvare i dati nel db "result"
         */
        global $wpdb;
        $table_name = $wpdb->prefix . $this->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("INSERT INTO $table_name SET 
            user_id = %d,
            test_id = %d,
            score_s1 = %d,
            score_s2 = %d,
            score_s3 = %d,
            score_s4 = %d,
            score_s5 = %d,
            score_s6 = %d,
            score_s7 = %d,
            score_s8 = %d,
            score_s9 = %d,
            score_s10 = %d,
            elements_s1 = %d,
            elements_s2 = %d,
            elements_s3 = %d,
            elements_s4 = %d,
            elements_s5 = %d,
            elements_s6 = %d,
            elements_s7 = %d,
            elements_s8 = %d,
            elements_s9 = %d,
            elements_s10 = %d,
            examination = %d,
            date = %s,
            date_gmt = %s
            ",
            $user_id,
            $test_id,
            $score_s1,
            $score_s2,
            $score_s3,
            $score_s4,
            $score_s5,
            $score_s6,
            $score_s7,
            $score_s8,
            $score_s9,
            $score_s10,
            $elements_s1,
            $elements_s2,
            $elements_s3,
            $elements_s4,
            $elements_s5,
            $elements_s6,
            $elements_s7,
            $elements_s8,
            $elements_s9,
            $elements_s10,
            $examination,
            $date,
            $date_gmt
        );

        $wpdb->query( $safe_sql );
        
    }
    
    /*
     * Given the test_id the type of the test (1,2,3,4) is returned
     * 
     * @param $test_id
     * @return int The type of the test
     */
    public function get_type_of_test($test_id){
        
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_test";
        $safe_sql = $wpdb->prepare("SELECT type FROM $table_name WHERE id = %d ", $test_id);
        $test_obj = $wpdb->get_row($safe_sql);

        return $test_obj->type;
        
    }
    
    /*
     * Given the test_id verifies if the test has been already completed by the
     * current user
     * 
     * @param $test_id
     * @return bool True if the test has been already completed by the current
     * user, False if the test hasn't been already completed by the current
     * user
     */
    public function examination_already_completed($test_id){
        
        $user_id = get_current_user_id();
        
        /*
         * Check in the 'result' db table if the current user has already
         * completed the specified test.
         * Returns true if the examination has been already completed, otherwise
         * returns false
         * 
         * @param $test_id The id of the test
         * @return bool
         */
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("SELECT id FROM $table_name WHERE examination = 1 and user_id = %d and test_id = %d", $user_id, $test_id);
        $test_obj = $wpdb->get_row($safe_sql);

        if($wpdb->num_rows > 0){
            return true;
        }else{
            return false;
        }
        
    }
    
    /*
     * Given a test id and the number of the section returns the section title.
     * 
     * @param $test_id
     * @param $section_number
     * @return string The title of the section
     */
    public function get_section_title_from_test_id($test_id, $section_number){
        
        //get the test object of the specified test id
        global $wpdb;
        $table_name = $wpdb->prefix . $this->get('slug') . "_test";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $test_id);
        $test_obj = $wpdb->get_row($safe_sql);
        
        //get the section id
        $section_id = $test_obj->{"section" . $section_number . "_id"};
        
        //get the title of the section
        $table_name = $wpdb->prefix . $this->get('slug') . "_section";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $section_id);
        $section_obj = $wpdb->get_row($safe_sql);
        
        if($wpdb->num_rows > 0){
            return $section_obj->title;
        }else{
            return false;
        }
        
    }
    
    /*
     * Given a test id the title of the test is returned
     * 
     * @param $test_id
     * @return string The title of the test
     */
    public function get_test_title($test_id){
        
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_test";
        $safe_sql = $wpdb->prepare("SELECT title FROM $table_name WHERE id = %d", $test_id);
        $test_obj = $wpdb->get_row($safe_sql);
        
        return $test_obj->title;
        
    }
    
    /*
     * Get the maximum number of sections available in the array of results
     * 
     * @param $results
     * @return int The number of sections used in the results
     */
    public function get_number_of_sections($results){
        
        /*
         * Initialize the array used to find which sections are used in the
         * results
         */
        $has_section = array();
        $has_section['1'] = false;
        $has_section['2'] = false;
        $has_section['3'] = false;
        $has_section['4'] = false;
        $has_section['5'] = false;
        $has_section['6'] = false;
        $has_section['7'] = false;
        $has_section['8'] = false;
        $has_section['9'] = false;
        $has_section['10'] = false;
        
        /*
         * Parse the results and set the specific section flag to True if a
         * section is used
         */
        foreach($results as $key => $result){
            
            global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_test";
            $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $result['test_id']);
            $test_obj = $wpdb->get_row($safe_sql);
            
            for($i=1;$i<=10;$i++){
                if($test_obj->{"section" . $i . "_id"} != 0){$has_section[ (string) $i ] = true;}
            }
            
        }
        
        /*
         * Count the number of used sections
         */
        $section_counter = 0;
        foreach($has_section as $single_has_section){
            if($single_has_section){$section_counter++;}
        }
        
        return $section_counter;
        
    }
    
    /*
     * Prevent the creation of empty sections followed by selected
     * sections and move all the sections to the first positions
     * 
     * @param $section[1-10]_id
     * @return array
     */
    public function prepare_sections($section1_id, $section2_id, $section3_id, $section4_id, $section5_id, $section6_id, $section7_id, $section8_id, $section9_id, $section10_id)
    {

        $sections_a = array();
        
        //add the sections to the $sections_a array
        for ($i = 1; $i <= 10; $i++) {

            if (intval(${"section" . $i . "_id"}, 10) > 0) {
                $sections_a[] = ${"section" . $i . "_id"};
            }

        }
        
        //fill with 0 all the sections to avoid php notices
        $number_of_empty_sections = ( 10 - count($sections_a) );
        for($i=1;$i <= $number_of_empty_sections;$i++){
            
            $sections_a[] = 0;
            
        }

        return $sections_a;

    } 
    
    /*
     * Prevent the creation of empty sentences followed by non-empty
     * sentences by moving all the sentences to the first positions
     * 
     * @param $sentence[1-10]
     * @return array
     */
    public function prepare_sentences($sentence1, $sentence2, $sentence3, $sentence4, $sentence5, $sentence6, $sentence7, $sentence8, $sentence9, $sentence10)
    {

        $sentences_a = array();
        
        //add the sentences to the $sentences_a array
        for ($i = 1; $i <= 10; $i++) {

            if (strlen(trim(${"sentence" . $i})) > 0) {
                $sentences_a[] = ${"sentence" . $i};
            }

        }
        
        //fill with empty strings all the sentences to avoid php notices
        $number_of_empty_sentences = ( 10 - count($sentences_a) );
        for($i=1;$i <= $number_of_empty_sentences;$i++){
            
            $sentences_a[] = '';
            
        }

        return $sentences_a;

    }
    
    /*
     * Get the number of sentences in a section.
     * 
     * @param $section_id
     * @return int the number of sentences
     */
    public function get_number_of_sentences($section_id){
        
        global $wpdb;
        $table_name = $wpdb->prefix . $this->get('slug') . "_section";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $section_id);
        $section_obj = $wpdb->get_row($safe_sql); 
        
        $number_of_sentences = 0;
        
        for($i=1;$i<=10;$i++){
            
            if( strlen( trim($section_obj->{"sentence" . $i} ) ) > 0 ){
                $number_of_sentences++;
            }
            
        }
        
        return $number_of_sentences;
        
    }
    
    /*
     * Generate the custom-[blog_id].css file based on the plugin options.
     */
    public function write_custom_css(){

        //turn on output buffering
        ob_start();

        //generate the CSS
        ?>
/* Base Font Size ----------------------------------------------------------- */
<?php $base_font_size = intval(get_option($this->get('slug'). '_base_font_size') , 10); ?>
.dagt-container input[type="text"],
.dagt-container input[type="text"]:focus,
.dagt-container select,
.dagt-submit-form,
.dagt-message-only-registered-students,
.dagt-message-examination-completed,
.dagt-message-examination-already-completed,
.dagt-section-description,
.dagt-test-description,
.dagt-sentence-container,
.dagt-additional-info,
.dagt-general-answers,
.dagt-section-result,
.ui-tooltip,
.dagt-container .tooltip > div,
.dagt-container .chosen-container-single .chosen-single span,
.chosen-container .chosen-results li.active-result,
.dagt-sentence-index,
.dagt-correct-answer-textual{
    font-size: <?php echo $base_font_size; ?>px !important;
}
.dagt-container h2{
    font-size: <?php echo $base_font_size * ( 28 / 13 ); ?>px !important;
}
.dagt-container h3{
    font-size: <?php echo $base_font_size * ( 16 / 13 ); ?>px !important;
}
/* Base Line Height --------------------------------------------------------- */
<?php $base_line_height = intval(get_option($this->get('slug'). '_base_line_height') , 10); ?>
.dagt-general-answers,
.dagt-section-result,
.dagt-message-only-registered-students,
.dagt-message-examination-completed,
.dagt-message-examination-already-completed,
.dagt-container h3,
.dagt-test-description,
.dagt-section-description,
.dagt-additional-info,
.dagt-sentence-container,
.ui-tooltip,
.dagt-container .tooltip > div{
    line-height: <?php echo $base_line_height; ?>px !important;
}
.ui-tooltip,
.dagt-container .tooltip > div{
    padding: <?php echo ( ( $base_line_height / 2 ) - 1 ); ?>px <?php echo ( ( $base_line_height / 2 ) - 1 ); ?>px !important;
}
.dagt-container .chosen-container-single .chosen-single div b {
    margin-top:  <?php echo ( $base_line_height - 2 ) / 2 - 2; ?>px !important;
}
.dagt-sentence-container{
    line-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
    min-height: <?php echo $base_line_height; ?>px !important;
}
.dagt-select-container{
    max-height: <?php echo $base_line_height; ?>px !important;
}
.dagt-message-only-registered-students,
.dagt-message-examination-completed,
.dagt-message-examination-already-completed{
    padding: <?php echo $base_line_height - 1; ?>px !important;
}
.dagt-section-container{
    margin: <?php echo $base_line_height * 2; ?>px 0 !important;
}
.dagt-additional-info{
    margin-bottom: <?php echo $base_line_height * 2; ?>px !important;
}
.dagt-section-description,
.dagt-container h3,
.dagt-section-result{
    margin-bottom: <?php echo $base_line_height; ?>px !important;
}
.dagt-container h2{
    line-height: <?php echo $base_line_height * ( 30 / 21 ); ?>px !important;
    margin-bottom: <?php echo $base_line_height * ( 12 / 21 ); ?>px !important;
}
.dagt-submit-form{
    height: <?php echo $base_line_height * ( 33 / 21 ); ?>px !important;
    line-height: <?php echo $base_line_height * ( 33 / 21 ); ?>px !important;
    padding: 0 <?php echo $base_line_height * ( 22 / 21 ); ?>px !important;
}
.dagt-ajax-loader{
    height: <?php echo $base_line_height * ( 33 / 21 ); ?>px !important;
    width: <?php echo $base_line_height * ( 33 / 21 ); ?>px !important;
}
.dagt-container select{
    height: <?php echo $base_line_height; ?>px !important;
    line-height: <?php echo $base_line_height * ( 17 / 21 ); ?>px !important;
}
.dagt-container input[type="text"],
.dagt-container input[type="text"]:focus{
    line-height: <?php echo $base_line_height * ( 20 / 21 ); ?>px !important;
}
.dagt-container input[type="text"].dagt-auto-line-height,
.dagt-container input[type="text"]:focus.dagt-auto-line-height{
    line-height: <?php echo $base_line_height * ( 20 / 21 ) - 1; ?>px !important;
}
.dagt-container .chosen-container-single .chosen-single{
    height: <?php echo $base_line_height; ?>px !important;
}
.dagt-container .chosen-container-single .chosen-single span{
    height: <?php echo ( $base_line_height - 2 ); ?>px !important;
    line-height: <?php echo ( $base_line_height - 2 ); ?>px !important;
}
.dagt-container .chosen-results .active-result{
    min-height: <?php echo $base_line_height; ?>px !important;
    line-height: <?php echo $base_line_height; ?>px !important;
}
.dagt-container .dagt-select-container{
    min-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
}
.dagt-container .dagt-correct-answer{
    max-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
    min-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
    line-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
}
.dagt-container .dagt-correct-answer-textual{
    min-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
    line-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
}
.dagt-container .dagt-info-tooltip{
    min-height: <?php echo ( $base_line_height ); ?>px !important;
    line-height: <?php echo ( $base_line_height ); ?>px !important;
    margin: <?php echo ( $base_line_height / 2 ); ?>px 0 <?php echo ( $base_line_height / 2 ); ?>px 0 !important;
}
.dagt-container .dagt-sentence-index{
    min-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
    line-height: <?php echo ( $base_line_height * 2 ); ?>px !important;
}
.dagt-container .chosen-container-single{
    margin-bottom: <?php echo $base_line_height * ( 2 / 21 ); ?>px !important;
}
.dagt-container .chosen-container .chosen-drop{
    top: <?php echo $base_line_height; ?>px !important;
}
.dagt-container .chosen-container-single{
    height: <?php echo $base_line_height; ?>px !important;
}
.dagt-container .chosen-container-single .chosen-single div b{
    background-position-y: <?php echo ( ( $base_line_height / 2 ) - 2.5 ); ?>px !important;
}
/* Base font Color ---------------------------------------------------------- */
.dagt-container input[type="text"],
.dagt-container input[type="text"]:focus,
.dagt-container select,
.dagt-message-only-registered-students,
.dagt-message-examination-completed,
.dagt-message-examination-already-completed,
.dagt-test-description,
.dagt-section-description,
.dagt-additional-info,
.dagt-general-answers,
.dagt-section-result,
.dagt-sentence-container,
.ui-tooltip,
.dagt-container .tooltip > div{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_base_font_color')); ?> !important;
}
/* Base Font Weight --------------------------------------------------------- */
.dagt-test-description,
.dagt-section-description,
.dagt-sentence-container,
.dagt-container input[type="text"],
.dagt-container input[type="text"]:focus,
.dagt-container select,
.dagt-additional-info,
.dagt-general-answers,
.dagt-section-result,
.chosen-container .chosen-results li.active-result,
.dagt-container .chosen-container-single a.chosen-single,
.ui-tooltip,
.dagt-container .tooltip > div{
    font-weight: <?php echo intval(get_option($this->get('slug'). '_base_font_weight'), 10); ?> !important;
}
/* Base Font Family --------------------------------------------------------- */
.dagt-test-description,
.dagt-section-description,
.dagt-sentence-container,
.dagt-additional-info,
.dagt-container select,
.dagt-section-result,
.dagt-general-answers,
.dagt-container input[type="text"],
.dagt-container input[type="text"]:focus,
.dagt-message-only-registered-students,
.dagt-message-examination-already-completed,
.dagt-message-examination-completed,
.dagt-submit-form,
.ui-tooltip,
.dagt-container .tooltip > div,
.dagt-container .chosen-results .active-result,
.dagt-container .chosen-container-single .chosen-single span,
.dagt-correct-answer-textual{
    font-family: <?php echo get_option($this->get('slug'). '_base_font_family'); ?> !important;
}
/* Test Title Font Color ---------------------------------------------------- */
.dagt-container h2{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_test_title_font_color')); ?> !important;
}
/* Headings Font Weight ----------------------------------------------------- */
.dagt-container h2,
.dagt-container h3{
    font-weight: <?php echo intval(get_option($this->get('slug'). '_headings_font_weight'), 10); ?> !important;
}
/* Headings Font Family ----------------------------------------------------- */
.dagt-container h2,
.dagt-container h3{
    font-family: <?php echo get_option($this->get('slug'). '_headings_font_family'); ?> !important;
}
/* Submit Button Background Color ------------------------------------------- */
.dagt-submit-form{
    background: <?php echo esc_attr(get_option($this->get('slug'). '_submit_button_background_color')); ?> !important;
}
/* Submit Button Background Color Hover ------------------------------------- */
.dagt-submit-form:hover{
    background: <?php echo esc_attr(get_option($this->get('slug'). '_submit_button_background_color_hover')); ?> !important;
}
/* Submit Button Font Color ------------------------------------------------- */
.dagt-submit-form,
.dagt-submit-form:hover{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_submit_button_font_color')); ?> !important;
}
/* Correct Answers Color ---------------------------------------------------- */
.dagt-correct-highlight{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_correct_answers_color')); ?> !important;
}
/* Wrong Answers Color ------------------------------------------------------ */
.dagt-wrong-highlight{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_wrong_answers_color')); ?> !important;
}
/* Corrected Answers Font Weight -------------------------------------------- */
.dagt-correct-answer-textual{
    font-weight: <?php echo intval(get_option($this->get('slug'). '_corrected_answers_font_weight'), 10); ?> !important;
}
/* Messages Background Color ------------------------------------------------ */
.dagt-message-only-registered-students,
.dagt-message-examination-completed,
.dagt-message-examination-already-completed{
    background: <?php echo esc_attr(get_option($this->get('slug'). '_messages_background_color')); ?> !important;
}
/* Messages Border Color ---------------------------------------------------- */
.dagt-message-only-registered-students,
.dagt-message-examination-completed,
.dagt-message-examination-already-completed{
    border-color: <?php echo esc_attr(get_option($this->get('slug'). '_messages_border_color')); ?> !important;
}
/* Messages Font Color ------------------------------------------------------ */
.dagt-message-only-registered-students,
.dagt-message-examination-completed,
.dagt-message-examination-already-completed{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_messages_font_color')); ?> !important;
}
/* Test Top Margin ---------------------------------------------------------- */
.dagt-container{
    margin-top: <?php echo intval(get_option($this->get('slug'). '_test_top_margin'), 10); ?>px !important;
}
/* Test Bottom Margin ------------------------------------------------------- */
.dagt-container{
    margin-bottom: <?php echo intval(get_option($this->get('slug'). '_test_bottom_margin'), 10); ?>px !important;
}
/* Section Title Font Color ------------------------------------------------- */
.dagt-container h3{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_section_title_font_color')); ?> !important;
}
/* Section Description Font Color ------------------------------------------- */
.dagt-section-description,
.dagt-section-result{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_section_description_font_color')); ?> !important;
}
/* Section Description Font Weight ------------------------------------------ */
.dagt-section-description,
.dagt-section-result{
    font-weight: <?php echo intval(get_option($this->get('slug'). '_section_description_font_weight'), 10); ?> !important;
}
/* Submit Button Font Weight ------------------------------------------------ */
.dagt-submit-form{
    font-weight: <?php echo intval(get_option($this->get('slug'). '_submit_button_font_weight'), 10); ?> !important;
}
/* Input Font Color --------------------------------------------------------- */
.dagt-container input[type="text"],
.dagt-container input[type="text"]:focus{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_input_font_color')); ?> !important;
}
/* Input Border Color ------------------------------------------------------- */
.dagt-container input[type="text"],
.dagt-container input[type="text"]:focus{
    border-color: <?php echo esc_attr(get_option($this->get('slug'). '_input_border_color')); ?> !important;
}
/* Select Font Color -------------------------------------------------------- */
.dagt-container .chosen-container-single .chosen-single span,
.dagt-container .chosen-results .active-result{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_select_font_color')); ?> !important;
}
/* Select Background Color -------------------------------------------------- */
.dagt-container .chosen-container,
.dagt-container .chosen-container-single,
.dagt-container .chosen-container-active,
.dagt-container .chosen-default,
.dagt-container .chosen-drop,
.dagt-container .chosen-single,
.dagt-container .chosen-search,
.dagt-container .chosen-results{
    background: <?php echo esc_attr(get_option($this->get('slug'). '_select_background_color')); ?> !important;
}
/* Select Border Color ------------------------------------------------------ */
.dagt-container .chosen-container-active.chosen-with-drop .chosen-single,
.dagt-container .chosen-container-single .chosen-single,
.dagt-container .chosen-container .chosen-drop{
    border-color: <?php echo esc_attr(get_option($this->get('slug'). '_select_border_color')); ?> !important;
}
/* Select Highlight Background Color ---------------------------------------- */
.dagt-container .chosen-container .chosen-results li.highlighted{
    background: <?php echo esc_attr(get_option($this->get('slug'). '_select_highlight_background_color')); ?> !important;
}
/* Select Highlight Font Color ---------------------------------------------- */
.dagt-container .chosen-container .chosen-results li.highlighted{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_select_highlight_font_color')); ?> !important;
}
/* Tooltip Font Color ------------------------------------------------------- */
.ui-tooltip,
.dagt-container .tooltip > div{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_tooltip_font_color')); ?> !important;
}
/* Tooltip Background Color ------------------------------------------------------- */
.ui-tooltip,
.dagt-container .tooltip > div{
    background: <?php echo esc_attr(get_option($this->get('slug'). '_tooltip_background_color')); ?> !important;
}
/* Tooltip Border Color ------------------------------------------------------- */
.ui-tooltip,
.dagt-container .tooltip > div{
    border-color: <?php echo esc_attr(get_option($this->get('slug'). '_tooltip_border_color')); ?> !important;
}
/* Dashicon Size ------------------------------------------------------------ */
.dagt-container .dashicons,
.dagt-container .dashicons-before:before{
    font-size: <?php echo esc_attr(get_option($this->get('slug'). '_dashicon_size')); ?>px !important;
}
/* Correct Answers Decoration ----------------------------------------------- */
<?php

switch( intval(get_option($this->get('slug'). '_correct_answers_decoration'), 10) ){

    case 1:

        echo '.dagt-correct-highlight > div{ text-decoration: none !important; }';
        break;

    case 2:

        echo '.dagt-correct-highlight > div{ text-decoration: underline !important; }';
        break;

    case 3:

        echo '.dagt-correct-highlight > div{ text-decoration: line-through !important; }';
        break;

}

?>
/* Wrong Answers Decoration ------------------------------------------------- */
<?php

switch( intval(get_option($this->get('slug'). '_wrong_answers_decoration'), 10) ){

    case 1:

        echo '.dagt-wrong-highlight > div{ text-decoration: none !important; }';
        break;

    case 2:

        echo '.dagt-wrong-highlight > div{ text-decoration: underline !important; }';
        break;

    case 3:

        echo '.dagt-wrong-highlight > div{ text-decoration: line-through !important; }';
        break;

}

?>
/* Input Horizontal Margin -------------------------------------------------- */
.dagt-container input[type="text"],
.dagt-container input[type="text"]:focus{
    margin-left: <?php echo intval(get_option($this->get('slug'). '_input_horizontal_margin'), 10); ?>px !important;
    margin-right: <?php echo intval(get_option($this->get('slug'). '_input_horizontal_margin'), 10); ?>px !important;
}
/* Select Horizontal Margin -------------------------------------------------- */
.dagt-container .chosen-container-single{
    margin-left: <?php echo intval(get_option($this->get('slug'). '_select_horizontal_margin'), 10); ?>px !important;
    margin-right: <?php echo intval(get_option($this->get('slug'). '_select_horizontal_margin'), 10); ?>px !important;
}
/* Enumeration Font Weight -------------------------------------------------- */
.dagt-sentence-index{
    font-weight: <?php echo intval(get_option($this->get('slug'). '_enumeration_font_weight'), 10); ?> !important;
}
/* Enumeration Font Color --------------------------------------------------- */
.dagt-sentence-index{
    color: <?php echo esc_attr(get_option($this->get('slug'). '_enumeration_font_color')); ?> !important;
}
/* Select Arrow Color ------------------------------------------------------- */
.dagt-container .chosen-container-single .chosen-single div b {
    <?php $select_arrow_color = str_replace('#', '', get_option($this->get('slug'). '_select_arrow_color')); ?>
    background: url('../../../public/assets/img/select-arrow.php?color=<?php echo esc_attr($select_arrow_color); ?>') no-repeat 0 0 !important;
}
        <?php

        
        $custom_css_string = ob_get_clean();

        return @file_put_contents( $this->get('dir') . 'public/assets/css/custom-' . get_current_blog_id() . '.css', $custom_css_string, LOCK_EX);

    }
    
    /*
     * Given a test_id generates the html of the test
     */
    public function generate_grammar_test_html($test_id){
        
        //get test data
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_test";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $test_id);
        $test_obj = $wpdb->get_row($safe_sql);

        //OUTPUT -----------------------------------------------------------

        //terminate if there is no test associated with the specified id
        if($test_obj === NULL){return '<p>' . esc_attr__('There is no test associated with this shortcode.', 'dagt') . '</p>';}

        /*
         * Don't show the test if the test type is one of the following:
         * 
         * - free practice (2)
         * - practice (3)
         * - examination (4)
         * 
         * And at the same time the user is not a student. ( namely he
         * doesn't have the 'dagt_perform_practice_and_examination'
         * capability )
         */
        if( ( ( intval($test_obj->type, 10) == 2 ) or
            ( intval($test_obj->type, 10) == 3 ) or
            ( intval($test_obj->type, 10) == 4 ) ) and
            ( !current_user_can('dagt_perform_practice_and_examination') )
        ){

            $output = '<div class="dagt-message-only-registered-students">' . esc_attr(get_option( $this->get('slug') . "_non_public_test_text")) . '</div>';

        }elseif(
            intval($test_obj->type, 10) == 4 and
            $this->examination_already_completed($test_obj->id)
        ){       

            $output = '<div class="dagt-message-examination-already-completed">' . esc_attr__('You have already completed this examination.', 'dagt') . '</div>';

        }else{

            //container
            $output = '<div id="dagt-container-' . intval($test_id, 10) . '" class="dagt-container" data-test-id="' . intval($test_id, 10) . '">';

                //test title
                if( intval(get_option($this->get('slug') . '_show_test_title'), 10) == 1){
                    $output .= '<h2 class="dagt-test-title">' . esc_attr(stripslashes($test_obj->title)) . '</h2>';
                }
                
                //test description
                $output .= '<div class="dagt-test-description">' . esc_attr(stripslashes($test_obj->description)) . '</div>';

                //output all the 10 sections
                for($i=1;$i<=10;$i++){

                    $section_id = $test_obj->{"section" . $i . "_id"};

                    //get the data about the considered section
                    $table_name = $wpdb->prefix . $this->get('slug') . "_section";
                    $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $section_id);
                    $section_obj = $wpdb->get_row($safe_sql);

                    if($section_obj !== NULL){

                        //section container start
                        $output .= '<div id="dagt-section-container-' . $test_obj->id . '-' . $section_obj->id .'" class="dagt-section-container" data-section-id="' . $section_obj->id . '" data-test-section-id="' . $i . '">';

                            //section title
                            $output .= '<h3>' . esc_attr(stripslashes($section_obj->title)) . '</h3>';

                            //section description
                            $output .= '<div class="dagt-section-description">' . esc_attr(stripslashes($section_obj->description)) . '</div>';

                            //output all the 10 sentences of the section
                            for($t=1;$t<=10;$t++){

                                //reset the element index
                                $this->element_index = 0;

                                $sentence = stripslashes($section_obj->{"sentence" . $t});

                                $sentence = $this->encode_sentence($sentence);
                                
                                //convert missing word
                                $sentence = $this->convert_missing_word($sentence);

                                //convert multiple selection
                                $sentence = $this->convert_multiple_selection($sentence);

                                /*
                                 * Add the sentence in the output only if it's
                                 * not empty
                                 */
                                if(strlen(trim($sentence)) > 0){
                                    
                                    /*
                                     * Add the enumeration only if the
                                     * enumeration value is equal to 1
                                     */
                                    if(intval($section_obj->enumerate, 10) == 1){
                                        $enumerate = '<div class="dagt-sentence-index">' . $t . '.</div>';
                                    }else{
                                        $enumerate = '';
                                    }
                                    
                                    $output .= '<div class="dagt-sentence-container" data-sentence-id="' . $t . '">' . $enumerate . ' ' . $sentence . '</div>';
                                    
                                }

                            }

                        //section container end
                        $output .= '</div>';

                    }

                }

                //test additional info
                if(strlen(trim($test_obj->additional_info)) > 0){
                    $output .= '<div class="dagt-additional-info">' . esc_attr(stripslashes($test_obj->additional_info)) . '</div>';
                }

                //test submit button
                $output .= '<div class="dagt-submit-form-container dagt-clearfix">';
                
                    $output .= '<input class="dagt-submit-form" data-type="check-answers" type="submit" value="' . esc_attr(get_option( $this->get('slug') . "_submit_button_text")) . '">';
                    $output .= '<input class="dagt-submit-form" data-type="reset-test" type="submit" value="' . esc_attr(get_option( $this->get('slug') . "_reset_button_text")) . '">';
                    
                    //ajax loader
                    $color = str_replace('#', '', get_option( $this->get('slug') . '_ajax_loader_color'));
                    $size = get_option( $this->get('slug') . '_base_line_height') * ( 33 / 21 );
                    $style = "background: url('" . $this->get('url') . "public/assets/img/ajax-loader.php?color=" . $color . "&size=" . $size . "') no-repeat 0 0 !important;";
                    $output .= '<div class="dagt-ajax-loader" style="' . $style . '"></div>';
                    
                    $output .= "</div>";
                    
            $output .= '</div>';//.dagt-container

        }
        
        return $output;
        
    }
    
    /*
     * Convert all the occurences of [mw id="x"] with the proper html
     * based on the data stored in the "missing_word" db table
     * 
     * @param $sentence The senctence that might include the [mw id="x"]
     * occurences
     * @return String The sentence with the [mw id="x"] occurences converted
     * with the proper html
     */
    private function convert_missing_word($sentence){
        
        $sentence = preg_replace_callback(
            '
            {\[mw
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
            array($this, 'preg_replace_callback_1'),
            $sentence
        );

        return $sentence;
        
    }
    
    private function convert_multiple_selection($sentence){
        
        $sentence = preg_replace_callback(
            '
            {\[ms
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
            array($this, 'preg_replace_callback_2'),
            $sentence
        );

        return $sentence;
        
    }
    
    /*
     * Callback of the preg_replace_callback() function
     * 
     * This callback is used to avoid an anonimus function as a parameter of the
     * preg_replace_callback() function for PHP backward compatibility
     * 
     * Look for uses of preg_replace_callback_6 to find which
     * preg_replace_callback() function is actually using this callback
     */
    public function preg_replace_callback_1($m){
        
        $missing_word_id = $m[1];
        
        //retrieve the missing_word data
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_missing_word";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $missing_word_id);
        $missing_word_obj = $wpdb->get_row($safe_sql);
        
        /*
         * Calculate the input field size, that should be the correct_answer
         * length plus a random number from 0 to 10 ( so the students can't
         * have a hint about the missing word by measuring the input text size )
         */
        $correct_answer_size = strlen(trim($missing_word_obj->correct_answer));
        
        //apply the "Random Input Offset" option
        $random_input_offset = intval(get_option($this->get('slug'). '_random_input_offset') , 10);
        $input_field_size = $correct_answer_size + rand(0, $random_input_offset);
        
        //do not allow an input size equal to 0, set 1 instead
        if($input_field_size == 0){$input_field_size = 1;}
        
        //generate the output
        $output = '<input data-element-index="' . $this->element_index . '" data-mw-id="' . $missing_word_id . '" type="text" value="" size="' . $input_field_size . '">';
            
        //increase the element index
        $this->element_index++;
        
        return $output;
            
    }
    
    /*
     * Callback of the preg_replace_callback() function
     * 
     * This callback is used to avoid an anonimus function as a parameter of the
     * preg_replace_callback() function for PHP backward compatibility
     * 
     * Look for uses of preg_replace_callback_6 to find which
     * preg_replace_callback() function is actually using this callback
     */
    public function preg_replace_callback_2($m){
        
        $multiple_selection_id = $m[1];
        
        //retrieve the missing_word data
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_multiple_selection";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d ", $multiple_selection_id);
        $missing_word_obj = $wpdb->get_row($safe_sql);
        
        //generate the output
        $output = '<div class="dagt-select-container">';
        
            $output .= '<select class="dagt-multiple-selection" data-element-index="' . $this->element_index . '" data-ms-id="' . $multiple_selection_id . '">';

            for($i=1;$i<=5;$i++){
                if( strlen(trim($missing_word_obj->{"alternative_" . $i})) > 0 ){
                    $output .= '<option value="' . $i . '">' . esc_attr(trim(stripslashes($missing_word_obj->{"alternative_" . $i}))) . '</option>';
                }
            }
        
            $output .= '</select>';
            
        $output .= '</div>';
        
        //increase the element index
        $this->element_index++;
        
        return $output;
            
    }
    
    /*
     * Verify if in the sentence there is at least one missing word. Returns
     * true if there is at least a missing word, return false if there are no
     * missing words
     * 
     * @param $sentence
     * @return bool
     */
    public function enable_missing_word_edit($sentence){
        
        $number_of_valid_ids = 0;
        
        preg_match_all(
            '
            {\[mw
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
            $sentence,
            $matches
        );
        
        $captures = $matches[1];
        foreach($captures as $key => $missing_word_id){
            
            global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_missing_word";
            $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $missing_word_id);
            $wpdb->get_results($safe_sql);
            
            if($wpdb->num_rows > 0){
                $number_of_valid_ids++;
            }
            
        }
        
        if($number_of_valid_ids > 0){
            return true;
        }else{
            return false;
        }
        
    }
    
    /*
     * Verify if in the sentence there is at least one multiple selection.
     * Returns true if there is at least a missing word, return false if there
     * are no missing words
     * 
     * @param $sentence
     * @return bool
     */
    public function enable_multiple_selection_edit($sentence){
        
        $number_of_valid_ids = 0;
        
        preg_match_all(
            '
            {\[ms
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
            $sentence,
            $matches
        );
        
        $captures = $matches[1];
        foreach($captures as $key => $multiple_selection_id){
            
            global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_multiple_selection";
            $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $multiple_selection_id);
            $wpdb->get_results($safe_sql);
            
            if($wpdb->num_rows > 0){
                $number_of_valid_ids++;
            }
            
        }
        
        if($number_of_valid_ids > 0){
            return true;
        }else{
            return false;
        }
        
    }
    
    /*
     * Verify that:
     * 
     * 1 - All the "mw" and "ms" included in the sentence exist
     * 2 - The sentence includes at least one "ms" or one "mw" 
     * 
     * Returns true if these two conditions are met, otherwise returns false
     * 
     * @param $sentence
     * @return bool
     */
    public function is_valid_sentence($sentence){
        
        //extract all the missing words identifier
        preg_match_all(
            '
            {\[mw
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
            $sentence,
            $matches
        );
        
        //return false if one of the missing words don't exist
        $captures_mw = $matches[1];
        foreach($captures_mw as $key => $missing_word_id){
            
            global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_missing_word";
            $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $missing_word_id);
            $wpdb->get_results($safe_sql);
            
            if($wpdb->num_rows == 0){
                return false;
            }
            
        }
        
        //extract all the multiple selections identifier
        preg_match_all(
            '
            {\[ms
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
            $sentence,
            $matches
        );
        
        //return false if one of the multiple selections don't exist
        $captures_ms = $matches[1];
        foreach($captures_ms as $key => $multiple_selection_id){
            
            global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_multiple_selection";
            $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $multiple_selection_id);
            $wpdb->get_results($safe_sql);
            
            if($wpdb->num_rows == 0){
                return false;
            }
            
        }
        
        //return false if in the sentence there are no mw or ms
        if(count($captures_mw) == 0 and count($captures_ms) == 0){return false;}
        
        /*
         * If we are here means that all the mw and ms exist, and also that this
         * sentence include at least on mw or ms, so return true
         */
        return true;
        
    }
    
    /*
     * Verify if a section is used in at least a test
     * 
     * Returns true if the section has been used in at least a test, otherwise
     * return false
     * 
     * @param $section_id
     * @return bool
     */
    public function is_section_used($section_id){
        
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_test";
        $test_a = $wpdb->get_results("SELECT * FROM $table_name", ARRAY_A);

        foreach($test_a as $key => $test){
            
            for($i=1;$i<=10;$i++){
                
                if($test["section" . $i . "_id"] == $section_id){return true;}
                
            }
            
        }
        
        return false;
        
    }
    
    /*
     * Verify if the given section exists
     * 
     * Returns true if the section exists, otherwise returns false
     * 
     * @param $section_id
     * @return bool
     */
    public function section_exists($section_id){
        
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_section";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE id = %d", $section_id);
        $wpdb->get_results($safe_sql);

        if($wpdb->num_rows > 0){
            return true;
        }else{
            return false;
        }
        
    }
    
    /*
     * Returns true if there are no results associated with a specific test,
     * otherwise returns false
     * 
     * @param $test_id
     * @return bool
     */
    public function no_test_results($test_id){
        
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE test_id = %d", $test_id);
        $wpdb->get_results($safe_sql);

        if($wpdb->num_rows > 0){
            return false;
        }else{
            return true;
        }
        
    }
    
    /*
     * Returns true if there are no results associated with a specific student,
     * otherwise returns false
     * 
     * @param $user_id
     * @return bool
     */
    public function no_student_results($user_id){
        
        global $wpdb; $table_name = $wpdb->prefix . $this->get('slug') . "_result";
        $safe_sql = $wpdb->prepare("SELECT * FROM $table_name WHERE user_id = %d", $user_id);
        $wpdb->get_results($safe_sql);

        if($wpdb->num_rows > 0){
            return false;
        }else{
            return true;
        }
        
    }
    
    /*
     * Encode the <, >, &, " and ' available in the sentence without replacing
     * the " present in the missing words and multiple selections shortcodes
     * 
     * @param $sentence The non-encoded sentence
     * @return string The encoded sentence 
     */
    function encode_sentence($sentence){
        
        /*
         * Remove the temporarely the double quotes inside the sentence from the
         * missing words and from the multiple sentences
         */
        $sentence = preg_replace_callback(
            '
            {\[mw
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
            array($this, 'preg_replace_callback_3'),
            $sentence
        );
        
        $sentence = preg_replace_callback(
            '
            {\[ms
            \s+
            id="
            (\d+)
            "\]
            }ixu', # Enable case insentive, extended and unicode modifiers
            array($this, 'preg_replace_callback_4'),
            $sentence
        );
        
        //encode the <, >, &, " and '
        $sentence = esc_attr($sentence);
        
        /*
         * Add the double quotes inside the sentence in the missing words and in
         * the multiple sentences
         */
        $sentence = preg_replace_callback(
            '
            {\[mw
            \s+
            id=
            (\d+)
            \]
            }ixu', # Enable case insentive, extended and unicode modifiers
            array($this, 'preg_replace_callback_5'),
            $sentence
        );
        
        $sentence = preg_replace_callback(
            '
            {\[ms
            \s+
            id=
            (\d+)
            \]
            }ixu', # Enable case insentive, extended and unicode modifiers
            array($this, 'preg_replace_callback_6'),
            $sentence
        );
        
        return $sentence;
        
    }
    
    /*
     * Callback of the preg_replace_callback() function
     * 
     * This callback is used to avoid an anonimus function as a parameter of the
     * preg_replace_callback() function for PHP backward compatibility
     * 
     * Look for uses of preg_replace_callback_3 to find which
     * preg_replace_callback() function is actually using this callback
     */
    public function preg_replace_callback_3($m){
        

        return '[mw id=' . $m[1] . ']';
            
    }
    
    /*
     * Callback of the preg_replace_callback() function
     * 
     * This callback is used to avoid an anonimus function as a parameter of the
     * preg_replace_callback() function for PHP backward compatibility
     * 
     * Look for uses of preg_replace_callback_4 to find which
     * preg_replace_callback() function is actually using this callback
     */
    public function preg_replace_callback_4($m){
        
        return '[ms id=' . $m[1] . ']';
        
    }
    
    /*
     * Callback of the preg_replace_callback() function
     * 
     * This callback is used to avoid an anonimus function as a parameter of the
     * preg_replace_callback() function for PHP backward compatibility
     * 
     * Look for uses of preg_replace_callback_5 to find which
     * preg_replace_callback() function is actually using this callback
     */
    public function preg_replace_callback_5($m){
        
        return '[mw id="' . $m[1] . '"]';
            
    }
    
    /*
     * Callback of the preg_replace_callback() function
     * 
     * This callback is used to avoid an anonimus function as a parameter of the
     * preg_replace_callback() function for PHP backward compatibility
     * 
     * Look for uses of preg_replace_callback_6 to find which
     * preg_replace_callback() function is actually using this callback
     */
    public function preg_replace_callback_6($m){
        
        return '[ms id="' . $m[1] . '"]';
            
    }

    /*
     * Returns the number of records available in the test db table
     *
     * @return int The number of records available in the test db table
     */
    public function get_number_of_tests(){

        global $wpdb;
        $table_name  = $wpdb->prefix . $this->get( 'slug' ) . "_test";
        $total_items = $wpdb->get_var( "SELECT COUNT(*) FROM $table_name" );

        return $total_items;

    }

    /*
     * Returns the number of records available in the section db table
     *
     * @return int The number of records available in the section db table
     */
    public function number_of_sections(){

        global $wpdb;
        $table_name  = $wpdb->prefix . $this->get( 'slug' ) . "_section";
        $total_items = $wpdb->get_var( "SELECT COUNT(*) FROM $table_name" );

        return $total_items;

    }

	/*
	 * Returns the number of registered students
	 *
	 * @return int The number of registered students
	*/
	public function get_number_of_students() {

	    $args = array( 'role' => 'dagt_student' );
	    $users = get_users($args);
	    return count($users);

	}
    
}